#Txhyy - Code.pyi

import os
import sys
from typing import Any, List


def getcwd() -> str:
    ...

def remove(path: str) -> None:
    ...

def removedir(path: str) -> None:
    ...

def cmd(command: str) -> int:
    ...

def psshell(command: str) -> None:
    ...

def chdir(path: str) -> None:
    ...

def mkdir(path: str) -> None:
    ...

def get_encoding(file_path: str) -> str:
    ...

def moencoding(input_file_path: str, output_file_path: str, new_encoding: str = 'utf-8') -> None:
    ...

def exists(path: str) -> bool:
    ...

def abspath(path: str) -> str:
    ...

def join(*paths: str) -> str:
    ...

def txprint(*objects: Any, sep: str = ' ', end: str = '\n', file = sys.stdout) -> None:
    ...

def cwdbytes() -> bytes:
    ...

def dirlist(path: str) -> List[str]:
    ...

def name(oldpath: str, newpath: str) -> None:
    ...

def file(path: str) -> None:
    ...

def idir(path: str) -> None:
    ...

READ = os.R_OK
WRITE = os.W_OK
RUN = os.X_OK

def pathq(filepath: str, r: int) -> bool:
    ...

def permissions(path: str, r: int) -> None:
    ...

import ctypes

class WinRightCilck:
    def auto_arrange(enable: bool) -> None:
        ...

    def grid(enable: bool) -> None:
        ...

    def icon(enable: bool) -> None:
        ...

    def desktop_icons(show: bool) -> None:
        ...

    def icon_size(size: int) -> None:
        ...

    def defaults() -> None:
        ...

    def sort_order(sort_type: int) -> None:
        ...


def _calculate_hash(data: bytes, algorithm: str) -> str:
    ...

def sha256(data: bytes) -> str:
    ...

def sha384(data: bytes) -> str:
    ...

def sha224(data: bytes) -> str:
    ...

def sha1(data: bytes) -> str:
    ...

def sha512(data: bytes) -> str:
    ...

def md5(data: bytes) -> str:
    ...

def blake2b(data: bytes) -> str:
    ...

def sha3_256(data: bytes) -> str:
    ...

def shake_128(data: bytes) -> str:
    ...

def shake_256(data: bytes) -> str:
    ...

def sha3_384(data: bytes) -> str:
    ...

def sha3_512(data: bytes) -> str:
    ...

def blake2s(data: bytes) -> str:
    ...

user32 = ctypes.windll.user32
shell32 = ctypes.windll.shell32
kernel32 = ctypes.windll.kernel32
ole32 = ctypes.windll.ole32
ws2_32 = ctypes.windll.ws2_32
winmm = ctypes.windll.winmm
gdi32 = ctypes.windll.gdi32
advapi32 = ctypes.windll.advapi32
odbc32 = ctypes.windll.odbc32
msimg32 = ctypes.windll.msimg32
_system32_path = os.path.join(os.environ['SystemRoot'], 'System32')
_winspool_path = os.path.join(_system32_path, 'winspool.drv')

winspool = ctypes.windll.LoadLibrary(_winspool_path)
setupapi = ctypes.windll.setupapi
crypt32 = ctypes.windll.crypt32
netapi32 = ctypes.windll.netapi32


class HTTPServer:
    """directory: index.html 访问"""
    def __init__(self, port: int = 8000, directory: str = '.') -> None:
        ...

    def start(self) -> None:
        ...


class HTTPSServer:
    """directory: index.html 访问"""
    def __init__(self, port: int = 4433, directory: str = '.', certfile: str = 'cert.pem', keyfile: str = 'key.pem') -> None:
        ...

    def start(self) -> None:
        ...


class BackendPythonServer:
    def __init__(self, python_file_path: str, port: int = None) -> None:
        ...

    def start(self) -> None:
        ...


class SelfSignedCertificateGenerator:
    def __init__(self, common_name: str, private_key_path: str = 'private_key.pem', certificate_path: str = 'certificate.pem',
                 key_size: int = 2048, validity_days: int = 365) -> None:
        ...

    def generate(self) -> None:
        ...


import ctypes
from win32com.client import Dispatch

FILE_ATTRIBUTE_HIDDEN = 0x2
FILE_ATTRIBUTE_READONLY = 0x1
FILE_ATTRIBUTE_ARCHIVE = 0x20
FILE_ATTRIBUTE_SYSTEM = 0x4
FILE_ATTRIBUTE_COMPRESSED = 0x800

def exit(code: int) -> None:
    ...

def attributes(file_path: str, hidden: bool = False, readonly: bool = False, archive: bool = False, system: bool = False, compressed: bool = False) -> None:
    ...

installlist = ['https://pypi.tuna.tsinghua.edu.cn/simple - 清华',"https://mirrors.aliyun.com/pypi/simple/ - 阿里云","https://pypi.mirrors.ustc.edu.cn/simple/ - 中国科学技术大学","https://pypi.doubanio.com/simple/ - 豆瓣镜像"]

def install(modules: List[str], mirror_url: str = "https://pypi.tuna.tsinghua.edu.cn/simple") -> str:
    ...

def quit(code: int) -> None:
    ...

def ctrlc(text: str) -> None:
    ...

def curani(cursor_path: str) -> None:
    ...

def size() -> tuple:
    ...

def copy(source_file: str, destination_file: str, TrueorFalse: bool = False) -> None:
    ...

def system(none = None) -> str:
    ...

def shortcut(target_path: str, shortcut_path: str, run_as_admin: bool = False) -> None:
    ...

shell = Dispatch('WScript.Shell')

def brightness(brightness: int) -> None:
    ...

Error = Exception

def desktop() -> str:
    ...

def backend(file: str) -> None:
    ...

def getcwd() -> str:
    ...

def username() -> str:
    ...

def increase(file_extension: str = None, file_type: str = None, icon_path: str = None, associated_program: str = None) -> None:
    ...

def delete(file_extension: str, file_type: str) -> None:
    ...

def modify(old_file_extension: str = None, old_file_type: str = None, new_file_extension: str = None, new_file_type: str = None,
           new_icon_path: str = None, new_associated_program: str = None) -> None:
    ...

def activate_windows(product_key: str) -> None:
    ...

Roaming = os.getenv('APPDATA')

def home() -> str:
    ...

def pi(numbers: int) -> float:
    ...

def factorial(n: int) -> int:
    ...

def sin(x: float, num_terms: int = 10) -> float:
    ...

def cos(x: float, num_terms: int = 10) -> float:
    ...

def tan(x: float, num_terms: int = 10) -> float:
    ...

def cot(x: float, num_terms: int = 10) -> float:
    ...

class ComplexNumber:
    def __init__(self, real: float, imag: float) -> None:
        ...

    def __add__(self, other: 'ComplexNumber') -> 'ComplexNumber':
        ...

    def __sub__(self, other: 'ComplexNumber') -> 'ComplexNumber':
        ...

    def __mul__(self, other: 'ComplexNumber') -> 'ComplexNumber':
        ...

    def __truediv__(self, other: 'ComplexNumber') -> 'ComplexNumber':
        ...

    def __str__(self) -> str:
        ...

def sqrt(a: float, tolerance: float = 1e-6, max_iterations: int = 100) -> float:
    ...

def exp(x: float, num_terms: int = 10) -> float:
    ...

def ln(x: float, tolerance: float = 1e-6, max_iterations: int = 100) -> float:
    ...

def power(x: float, n: int) -> float:
    ...

def combination(n: int, k: int) -> int:
    ...


import os

WXPYTHON = "wxpython"
PYQT6 = "pyqt6"
TKINTER = "tkinter"
PYQT5 = "pyqt5"

wxpython = "wxpython"
PyQt6 = "pyqt6"
Tkinter = "tkinter"
PyQt5 = "pyqt5"

class FileDialogIntegrator:
    def __init__(self, preferred_lib='auto', open_title="Open File", save_title="Save File",
                 filetypes=[("All Files", "*.*"), ("Text Files", "*.txt"), ("Python 源文件", "*.py")], initial_dir=None):
        self.preferred_lib = preferred_lib
        self.open_title = open_title
        self.save_title = save_title
        self.filetypes = filetypes
        self.initial_dir = initial_dir
        self.strategy = self._select_strategy()

    def _select_strategy(self):
        if self.preferred_lib == 'auto':
            # 按优先级尝试选择可用的库
            for lib in ['pyqt6', 'pyqt5', 'tkinter', 'wxpython']:
                try:
                    return self._get_strategy(lib)
                except ImportError:
                    continue
            raise ImportError("No supported GUI library found.")
        else:
            return self._get_strategy(self.preferred_lib)

    def _get_strategy(self, lib):
        if lib == 'tkinter':
            import tkinter as tk
            from tkinter import filedialog

            class TkinterFileDialogStrategy:
                def __init__(self, open_title, save_title, filetypes, initial_dir):
                    self.open_title = open_title
                    self.save_title = save_title
                    self.filetypes = filetypes
                    self.initial_dir = initial_dir

                def open_file_dialog(self):
                    root = tk.Tk()
                    root.withdraw()
                    # 居中显示主窗口（虽然 filedialog 依赖主窗口位置，但不一定能完全保证居中）
                    root.update_idletasks()
                    width = root.winfo_width()
                    height = root.winfo_height()
                    x = (root.winfo_screenwidth() // 2) - (width // 2)
                    y = (root.winfo_screenheight() // 2) - (height // 2)
                    root.geometry(f'{width}x{height}+{x}+{y}')

                    file_path = filedialog.askopenfilename(
                        title=self.open_title,
                        filetypes=self.filetypes,
                        initialdir=self.initial_dir
                    )
                    root.destroy()
                    return file_path

                def save_file_dialog(self):
                    root = tk.Tk()
                    root.withdraw()
                    # 居中显示主窗口
                    root.update_idletasks()
                    width = root.winfo_width()
                    height = root.winfo_height()
                    x = (root.winfo_screenwidth() // 2) - (width // 2)
                    y = (root.winfo_screenheight() // 2) - (height // 2)
                    root.geometry(f'{width}x{height}+{x}+{y}')

                    file_path = filedialog.asksaveasfilename(
                        title=self.save_title,
                        defaultextension=".txt",
                        filetypes=self.filetypes,
                        initialdir=self.initial_dir
                    )
                    root.destroy()
                    return file_path

            return TkinterFileDialogStrategy(self.open_title, self.save_title, self.filetypes, self.initial_dir)

        elif lib == 'pyqt6':
            from PyQt6.QtWidgets import QApplication, QFileDialog
            import sys

            class PyQt6FileDialogStrategy:
                def __init__(self, open_title, save_title, filetypes, initial_dir):
                    self.open_title = open_title
                    self.save_title = save_title
                    self.filetypes = self._convert_filetypes(filetypes)
                    self.initial_dir = initial_dir

                def _convert_filetypes(self, filetypes):
                    return ';;'.join([f'{name} ({pattern})' for name, pattern in filetypes])

                def open_file_dialog(self):
                    app = QApplication(sys.argv)
                    dialog = QFileDialog()
                    dialog.setWindowTitle(self.open_title)
                    dialog.setNameFilter(self.filetypes)
                    if self.initial_dir:
                        dialog.setDirectory(self.initial_dir)

                    # 居中显示对话框
                    screen_geometry = app.primaryScreen().geometry()
                    dialog_size = dialog.sizeHint()
                    x = (screen_geometry.width() - dialog_size.width()) // 2
                    y = (screen_geometry.height() - dialog_size.height()) // 2
                    dialog.move(x, y)

                    if dialog.exec():
                        file_path = dialog.selectedFiles()[0]
                    else:
                        file_path = None
                    app.quit()
                    return file_path

                def save_file_dialog(self):
                    app = QApplication(sys.argv)
                    dialog = QFileDialog()
                    dialog.setWindowTitle(self.save_title)
                    dialog.setNameFilter(self.filetypes)
                    if self.initial_dir:
                        dialog.setDirectory(self.initial_dir)

                    # 居中显示对话框
                    screen_geometry = app.primaryScreen().geometry()
                    dialog_size = dialog.sizeHint()
                    x = (screen_geometry.width() - dialog_size.width()) // 2
                    y = (screen_geometry.height() - dialog_size.height()) // 2
                    dialog.move(x, y)

                    if dialog.exec():
                        file_path = dialog.selectedFiles()[0]
                    else:
                        file_path = None
                    app.quit()
                    return file_path

            return PyQt6FileDialogStrategy(self.open_title, self.save_title, self.filetypes, self.initial_dir)

        elif lib == 'pyqt5':
            from PyQt5.QtWidgets import QApplication, QFileDialog
            import sys

            class PyQt5FileDialogStrategy:
                def __init__(self, open_title, save_title, filetypes, initial_dir):
                    self.open_title = open_title
                    self.save_title = save_title
                    self.filetypes = self._convert_filetypes(filetypes)
                    self.initial_dir = initial_dir

                def _convert_filetypes(self, filetypes):
                    return ';;'.join([f'{name} ({pattern})' for name, pattern in filetypes])

                def open_file_dialog(self):
                    app = QApplication(sys.argv)
                    dialog = QFileDialog()
                    dialog.setWindowTitle(self.open_title)
                    dialog.setNameFilter(self.filetypes)
                    if self.initial_dir:
                        dialog.setDirectory(self.initial_dir)

                    # 居中显示对话框
                    screen_geometry = app.desktop().screenGeometry()
                    dialog_size = dialog.sizeHint()
                    x = (screen_geometry.width() - dialog_size.width()) // 2
                    y = (screen_geometry.height() - dialog_size.height()) // 2
                    dialog.move(x, y)

                    if dialog.exec_():
                        file_path = dialog.selectedFiles()[0]
                    else:
                        file_path = None
                    app.quit()
                    return file_path

                def save_file_dialog(self):
                    app = QApplication(sys.argv)
                    dialog = QFileDialog()
                    dialog.setWindowTitle(self.save_title)
                    dialog.setNameFilter(self.filetypes)
                    if self.initial_dir:
                        dialog.setDirectory(self.initial_dir)

                    # 居中显示对话框
                    screen_geometry = app.desktop().screenGeometry()
                    dialog_size = dialog.sizeHint()
                    x = (screen_geometry.width() - dialog_size.width()) // 2
                    y = (screen_geometry.height() - dialog_size.height()) // 2
                    dialog.move(x, y)

                    if dialog.exec_():
                        file_path = dialog.selectedFiles()[0]
                    else:
                        file_path = None
                    app.quit()
                    return file_path

            return PyQt5FileDialogStrategy(self.open_title, self.save_title, self.filetypes, self.initial_dir)

        elif lib == 'wxpython':
            import wx

            class WxPythonFileDialogStrategy:
                def __init__(self, open_title, save_title, filetypes, initial_dir):
                    self.open_title = open_title
                    self.save_title = save_title
                    self.filetypes = self._convert_filetypes(filetypes)
                    self.initial_dir = initial_dir

                def _convert_filetypes(self, filetypes):
                    return '|'.join([f'{name}|{pattern}' for name, pattern in filetypes])

                def open_file_dialog(self):
                    app = wx.App()
                    frame = wx.Frame(None, -1, self.open_title)
                    frame.SetSize(0, 0, 200, 50)
                    dialog = wx.FileDialog(frame, self.open_title,
                                           defaultDir=self.initial_dir if self.initial_dir else "",
                                           wildcard=self.filetypes, style=wx.FD_OPEN)
                    if dialog.ShowModal() == wx.ID_OK:
                        file_path = dialog.GetPath()
                    else:
                        file_path = None
                    dialog.Destroy()
                    app.Destroy()
                    return file_path

                def save_file_dialog(self):
                    app = wx.App()
                    frame = wx.Frame(None, -1, self.save_title)
                    frame.SetSize(0, 0, 200, 50)
                    dialog = wx.FileDialog(frame, self.save_title,
                                           defaultDir=self.initial_dir if self.initial_dir else "",
                                           wildcard=self.filetypes, style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
                    if dialog.ShowModal() == wx.ID_OK:
                        file_path = dialog.GetPath()
                    else:
                        file_path = None
                    dialog.Destroy()
                    app.Destroy()
                    return file_path

            return WxPythonFileDialogStrategy(self.open_title, self.save_title, self.filetypes, self.initial_dir)
        else:
            raise ValueError(f"Unsupported library: {lib}")

    def open_file(self):
        return self.strategy.open_file_dialog()

    def save_file(self):
        return self.strategy.save_file_dialog()

if __name__ == "__main__":
    dialog = FileDialogIntegrator(
        preferred_lib=PyQt6,
        open_title="Select a Python file",
        save_title="Save your Python script",
        filetypes=[("Python Files", "*.py"), ("All Files", "*.*")],
        initial_dir=os.getcwd()
    )

    open_path = dialog.open_file()
    if open_path:
        print(f"Selected file to open: {open_path}")
    else:
        print("No file selected for opening.")

    save_path = dialog.save_file()
    if save_path:
        print(f"File will be saved at: {save_path}")
    else:
        print("No save path selected.")


import wmi
import ctypes
import winreg
import os, sys
import win32con
import win32gui
import win32api
import subprocess
from hashlib import *
import win32clipboard
from win32com.client import Dispatch


FILE_ATTRIBUTE_HIDDEN = 0x2
FILE_ATTRIBUTE_READONLY = 0x1
FILE_ATTRIBUTE_ARCHIVE = 0x20
FILE_ATTRIBUTE_SYSTEM = 0x4
FILE_ATTRIBUTE_COMPRESSED = 0x800


def exit(code):
    raise SystemExit(code)


def attributes(file_path, hidden=False, readonly=False, archive=False, system=False, compressed=False):
    """
    设置指定文件的属性为隐藏、只读、存档、系统或压缩
    :param file_path: 文件路径
    :param hidden: 是否设置为隐藏，默认为False
    :param readonly: 是否设置为只读，默认为False
    :param archive: 是否设置为存档，默认为False
    :param system: 是否设置为系统，默认为False
    :param compressed: 是否设置为压缩，默认为False
    """
    if os.name == 'nt':
        if os.path.exists(file_path):
            attributes = ctypes.windll.kernel32.GetFileAttributesW(file_path)
            if hidden:
                attributes |= FILE_ATTRIBUTE_HIDDEN
            else:
                attributes &= ~FILE_ATTRIBUTE_HIDDEN
            if readonly:
                attributes |= FILE_ATTRIBUTE_READONLY
            else:
                attributes &= ~FILE_ATTRIBUTE_READONLY
            if archive:
                attributes |= FILE_ATTRIBUTE_ARCHIVE
            else:
                attributes &= ~FILE_ATTRIBUTE_ARCHIVE
            if system:
                attributes |= FILE_ATTRIBUTE_SYSTEM
            else:
                attributes &= ~FILE_ATTRIBUTE_SYSTEM
            if compressed:
                attributes |= FILE_ATTRIBUTE_COMPRESSED
            else:
                attributes &= ~FILE_ATTRIBUTE_COMPRESSED
            ctypes.windll.kernel32.SetFileAttributesW(file_path, attributes)
        else:
            print('文件未找着')
            exit(1)
    else:
        print('attributes: 此功能只支持Windows系统! ')
        exit(1)


def quit(code):
    exit(code)


def ctrlc(text):
    """复制指定的内容"""
    win32clipboard.OpenClipboard()
    win32clipboard.EmptyClipboard()
    win32clipboard.SetClipboardData(win32clipboard.CF_UNICODETEXT, text)
    win32clipboard.CloseClipboard()


def curani(cursor_path):
    """设置鼠标指针(.cur,.ani)"""
    hCursor = win32gui.LoadImage(0, cursor_path, win32con.IMAGE_CURSOR, 0, 0, win32con.LR_LOADFROMFILE)
    if hCursor:
        win32api.SetSystemCursor(hCursor, win32con.OCR_NORMAL)


def size():
    """获取屏幕分辨率"""
    screen_width = win32api.GetSystemMetrics(0)
    screen_height = win32api.GetSystemMetrics(1)
    return screen_width, screen_height


def copy(source_file, destination_file, TrueorFalse=False):
    """
    复制文件,且不用管理员权限(如果文件是管理员权限或更高权限复制需要管理员权限)
    :param source_file:源文件路径,例如 "C:/path/to/source.txt"
    :param destination_file:目标文件路径,例如 "C:/path/to/destination.txt"
    :param TrueorFalse:是否覆盖文件,默认不覆盖.False:不覆盖,True:覆盖
    """
    if os.path.exists(destination_file):
        if not TrueorFalse:
            print(f"目标文件 {destination_file} 已存在，且不允许覆盖，跳过复制操作。")
            return
    try:
        source_hash = sha256(source_file)
        temp_file = destination_file + '.tmp'
        chunk_size = 4096
        with open(source_file, 'rb') as src_file:
            with open(temp_file, 'wb') as dest_file:
                while True:
                    chunk = src_file.read(chunk_size)
                    if not chunk:
                        break
                    dest_file.write(chunk)
        temp_hash = sha256(temp_file)
        if source_hash == temp_hash:
            os.replace(temp_file, destination_file)
        else:
            print("文件复制完成，但验证失败：内容不匹配。")
            os.remove(temp_file)
    except FileNotFoundError:
        raise Exception('copy2 Error: 复制失败')
    except PermissionError:
        if os.path.exists(temp_file):
            os.remove(temp_file)
        raise Exception('copy2 Error: 没有权限')
    except Exception as e:
        if os.path.exists(temp_file):
            os.remove(temp_file)
        raise Exception('copy2 Error:',e)
    except Exception as e:
        if os.path.exists(temp_file):
            os.remove(temp_file)
        raise Exception('copy2 Error:',e)
    

def system(none = None):
    """通过调用系统命令识别操作系统"""
    if none == None:
        try:
            result = subprocess.run("ver", capture_output=True, text=True, shell=True)
            if "Microsoft" in result.stdout:
                return "Windows"
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Darwin" in result.stdout:
                return "macOS"
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Linux" in result.stdout:
                return "Linux"
        except Exception as e:
            return f"发生错误: {e}"
        return "未知操作系统"
    else:
        return none
    

def shortcut(target_path, shortcut_path, run_as_admin=False):
    """
    创建指定文件的快捷方式
    :param target_path: 目标文件路径
    :param shortcut_path: 快捷方式保存路径
    :param run_as_admin: 是否以管理员权限运行，默认为False
    """
    shell = Dispatch('WScript.Shell')
    shortcut = shell.CreateShortCut(shortcut_path)
    shortcut.Targetpath = target_path
    if run_as_admin:
        shortcut.WorkingDirectory = os.path.dirname(target_path)
        shortcut.RunAsAdmin = True
    shortcut.save()


def brightness(brightness):
    """
    设置笔记本屏幕亮度
    brightness:设置亮度的百分比
    """
    if system() == 'Windows':
        try:
            c = wmi.WMI(namespace='wmi')
            methods = c.WmiMonitorBrightnessMethods()[0]
            methods.WmiSetBrightness(brightness, 0)
        except Exception as e:
            print(f"Error: {e}")
            sys.exit()
    elif system() == 'Linux':
        try:
            backlight_dir = '/sys/class/backlight'
            controllers = os.listdir(backlight_dir)
            if not controllers:
                print("未找到背光控制器")
                return
            controller = controllers[0]
            max_brightness_path = os.path.join(backlight_dir, controller, 'max_brightness')
            with open(max_brightness_path, 'r') as f:
                max_brightness = int(f.read().strip())
            brightness_value = int(max_brightness * (brightness / 100))
            brightness_path = os.path.join(backlight_dir, controller, 'brightness')
            with open(brightness_path, 'w') as f:
                f.write(str(brightness_value))
        except Exception as e:
            print(f"Error: {e}")
            sys.exit()
    elif system() == 'macOS':
        try:
            subprocess.run(['brightness', str(brightness / 100)], check=True)
        except subprocess.CalledProcessError as e:
            print(f"Error: {e}")
            sys.exit()
    else:
        print("不支持的操作系统")


Error = Exception


def desktop():
    """获取桌面位置(Desktop)"""
    return os.path.join(os.path.expanduser("~"), "Desktop")


def backend(file):
    """将指定的文件在后台运行"""
    subprocess.Popen(file, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, stdin=subprocess.DEVNULL)


def getcwd():
    """get? 获取当前工作目录"""
    if system() == 'Windows':
        kernel32 = ctypes.WinDLL('kernel32')
        buffer = ctypes.create_unicode_buffer(260)
        kernel32.GetCurrentDirectoryW(260, buffer)
        return buffer.value
    else:
        libc = ctypes.CDLL(ctypes.util.find_library('c'))
        buffer = ctypes.create_string_buffer(4096)
        libc.getcwd(buffer, 4096)
        return buffer.value.decode('utf-8')
    

def username():
    """获取用户名"""
    if system() == "Windows":
        return os.getenv("USERNAME")
    else:
        return os.getenv("USER")
    

def increase(file_extension=None, file_type=None, icon_path=None, associated_program=None):
    """
    :param file_extension 自定义后缀名
    :param file_type 自定义文件类型名称
    :param icon_path 图标文件路径，确保图标文件存在
    :param associated_program 关联的程序路径
    """
    if all(arg is None for arg in [file_extension, file_type, icon_path, associated_program]):
        print("Error: 至少需要提供一个参数")
        return
    try:
        if system() == 'Windows':
            try:
                if file_type:
                    key = winreg.CreateKey(winreg.HKEY_CLASSES_ROOT, file_type)
                    winreg.SetValue(key, "", winreg.REG_SZ, "Custom File")
                if icon_path and file_type:
                    icon_key = winreg.CreateKey(key, "DefaultIcon")
                    winreg.SetValue(icon_key, "", winreg.REG_SZ, icon_path)
                if associated_program and file_type:
                    shell_key = winreg.CreateKey(key, r"shell\open\command")
                    winreg.SetValue(shell_key, "", winreg.REG_SZ, f'"{associated_program}" "%1"')
                if file_extension and file_type:
                    ext_key = winreg.CreateKey(winreg.HKEY_CLASSES_ROOT, file_extension)
                    winreg.SetValue(ext_key, "", winreg.REG_SZ, file_type)
            except Exception as e:
                print(f"Error: {e}")
            finally:
                if 'key' in locals():
                    winreg.CloseKey(key)
                if 'icon_key' in locals():
                    winreg.CloseKey(icon_key)
                if 'shell_key' in locals():
                    winreg.CloseKey(shell_key)
                if 'ext_key' in locals():
                    winreg.CloseKey(ext_key)
        elif system() == 'macOS':
            if file_extension and associated_program:
                try:
                    subprocess.run(["duti", "-s", associated_program, file_extension], check=True)
                except subprocess.CalledProcessError as e:
                    print(f"Error: {e}")
        elif system() == 'Linux':
            if file_extension and associated_program:
                mime_type = subprocess.run(['xdg-mime', 'query', 'filetype', f'test{file_extension}'], capture_output=True, text=True).stdout.strip()
                try:
                    with open(os.path.expanduser('~/.local/share/applications/mimeapps.list'), 'a') as f:
                        f.write(f'[Default Applications]\n{mime_type}={associated_program}\n')
                except Exception as e:
                    print(f"Error: {e}")
    except Exception as e:
        raise Exception('increase Error:',e)
    

def delete(file_extension, file_type):
    """
    删除文件扩展名关联的函数
    :param file_extension: 要删除关联的文件扩展名
    :param file_type: 要删除关联的文件类型
    """
    if system() == 'Windows':
        try:
            try:
                ext_key = winreg.OpenKey(winreg.HKEY_CLASSES_ROOT, file_extension, 0, winreg.KEY_ALL_ACCESS)
                winreg.DeleteKey(ext_key, "")
                winreg.CloseKey(ext_key)
            except FileNotFoundError:
                pass
            try:
                key = winreg.OpenKey(winreg.HKEY_CLASSES_ROOT, file_type, 0, winreg.KEY_ALL_ACCESS)
                winreg.DeleteKey(key, "")
                winreg.CloseKey(key)
            except FileNotFoundError:
                raise Exception('delete Error: 文件不存在')
        except Exception as e:
            print(f"Error: {e}")
    elif system() == 'macOS':
        if file_extension:
            try:
                subprocess.run(["duti", "-d", file_extension], check=True)
            except subprocess.CalledProcessError as e:
                raise Exception('delete Error:',e)
    elif system() == 'Linux':
        if file_extension:
            mime_type = subprocess.run(['xdg-mime', 'query', 'filetype', f'test{file_extension}'], capture_output=True, text=True).stdout.strip()
            try:
                with open(os.path.expanduser('~/.local/share/applications/mimeapps.list'), 'r') as f:
                    lines = f.readlines()
                with open(os.path.expanduser('~/.local/share/applications/mimeapps.list'), 'w') as f:
                    for line in lines:
                        if not line.startswith(f'{mime_type}='):
                            f.write(line)
            except Exception as e:
                raise Exception('delete Error:',e)
            

def modify(old_file_extension=None, old_file_type=None, new_file_extension=None, new_file_type=None,
           new_icon_path=None, new_associated_program=None):
    """修改 文件后缀名的关联 """
    new_params = [new_file_extension, new_file_type, new_icon_path, new_associated_program]
    if all(arg is None for arg in new_params):
        print("Error: 至少需要提供一个新参数进行修改")
        return
    if system() == 'Windows':
        try:
            if old_file_extension and old_file_type:
                try:
                    ext_key = winreg.OpenKey(winreg.HKEY_CLASSES_ROOT, old_file_extension, 0, winreg.KEY_ALL_ACCESS)
                    winreg.DeleteKey(ext_key, "")
                    winreg.CloseKey(ext_key)
                except FileNotFoundError:
                    pass
                try:
                    key = winreg.OpenKey(winreg.HKEY_CLASSES_ROOT, old_file_type, 0, winreg.KEY_ALL_ACCESS)
                    winreg.DeleteKey(key, "")
                    winreg.CloseKey(key)
                except FileNotFoundError:
                    pass
            increase(new_file_extension, new_file_type, new_icon_path, new_associated_program)
        except Exception as e:
            raise Exception('modify Error:',e)
    elif system() == 'macOS':
        if old_file_extension:
            try:
                subprocess.run(["duti", "-d", old_file_extension], check=True)
            except subprocess.CalledProcessError as e:
                raise Exception('modify Error:',e)
        increase(new_file_extension, new_file_type, new_icon_path, new_associated_program)
    elif system() == 'Linux':
        if old_file_extension:
            mime_type = subprocess.run(['xdg-mime', 'query', 'filetype', f'test{old_file_extension}'], capture_output=True, text=True).stdout.strip()
            try:
                with open(os.path.expanduser('~/.local/share/applications/mimeapps.list'), 'r') as f:
                    lines = f.readlines()
                with open(os.path.expanduser('~/.local/share/applications/mimeapps.list'), 'w') as f:
                    for line in lines:
                        if not line.startswith(f'{mime_type}='):
                            f.write(line)
            except Exception as e:
                raise Exception('modify Error:',e)
        increase(new_file_extension, new_file_type, new_icon_path, new_associated_program)


# _txhyy.pyi
import wmi
import ctypes
import winreg
import hashlib
import os
import sys
import win32con
import win32gui
import win32api
import subprocess
import win32clipboard
from win32com.client import Dispatch

FILE_ATTRIBUTE_HIDDEN = 0x2
FILE_ATTRIBUTE_READONLY = 0x1
FILE_ATTRIBUTE_ARCHIVE = 0x20
FILE_ATTRIBUTE_SYSTEM = 0x4
FILE_ATTRIBUTE_COMPRESSED = 0x800

def exit(code: int) -> None: ...

def attributes(file_path: str, hidden: bool = False, readonly: bool = False, archive: bool = False,
               system: bool = False, compressed: bool = False) -> None: ...

installlist: list[str]

def _install_modules(modules: list[str], mirror_url: str = "https://pypi.tuna.tsinghua.edu.cn/simple") -> str: ...

def quit(code: int) -> None: ...

def ctrlc(text: str) -> None: ...

def curani(cursor_path: str) -> None: ...

def size() -> None: ...

def copy(source_file: str, destination_file: str, TrueorFalse: bool = False) -> None: ...

def system(none = None) -> None: ...

def shortcut(target_path: str, shortcut_path: str, run_as_admin: bool = False) -> None: ...

shell: Dispatch

def brightness(brightness: int) -> None: ...

Error: Exception

def desktop() -> None: ...

def backend(file: str) -> None: ...

def getcwd() -> None: ...

def username() -> None: ...

def increase(file_extension: str = None, file_type: str = None, icon_path: str = None,
             associated_program: str = None) -> None: ...

def delete(file_extension: str, file_type: str) -> None: ...

def modify(old_file_extension: str = None, old_file_type: str = None, new_file_extension: str = None,
           new_file_type: str = None, new_icon_path: str = None, new_associated_program: str = None) -> None: ...

def activate_windows(product_key: str) -> None: ...

Roaming: str

def home() -> str: ...

def _TEST() -> None: ...

install: callable
attr: callable


import re

def remove_jing(input_file_path, output_file_path):
    """去除#号"""
    try:
        with open(input_file_path, 'r', encoding='utf-8') as file:
            lines = file.readlines()

        new_lines = []
        for line in lines:
            stripped_line = line.lstrip()
            if not stripped_line.startswith('#'):
                line = re.sub(r'\s*#.*$', '', line)
                if line.strip():
                    new_lines.append(line)

        with open(output_file_path, 'w', encoding='utf-8') as file:
            file.writelines(new_lines)

    except FileNotFoundError:
        print(f"文件 {input_file_path} 未找到。")


def remove_ying(file_path):
    #去除三引号(''''''和"""""")
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()

        # 去除三引号注释块
        triple_quote_pattern = r'("""[\s\S]*?""")|(\'\'\'[\s\S]*?\'\'\')'
        content = re.sub(triple_quote_pattern, '', content)

        # 去除纯注释行和行内注释
        lines = content.splitlines()
        new_lines = []
        for line in lines:
            stripped_line = line.lstrip()
            if not stripped_line.startswith('#'):
                line = re.sub(r'\s*#.*$', '', line)
                if line.strip():
                    new_lines.append(line)

        new_content = '\n'.join(new_lines)

        with open(file_path, 'w', encoding='utf-8') as file:
            file.write(new_content)

        print(f"已成功处理文件 {file_path}，去除注释和三引号注释块。")
    except FileNotFoundError:
        print(f"文件 {file_path} 未找到。")


import argparse
import requests
import tarfile
import os
import shutil
from tqdm.rich import tqdm
from colorama import init, Fore
from packaging import version

# 初始化 colorama
init(autoreset=True)


def download_file(url, filename):
    """
    Download the file and display a colored progress bar using tqdm.rich
    """
    response = requests.get(url, stream=True)
    total_size = int(response.headers.get('content-length', 0))
    block_size = 1024

    with open(filename, 'wb') as file, tqdm(
            desc=f"Downloading {filename}",
            total=total_size,
            unit='iB',
            unit_scale=True,
    ) as progress_bar:
        for data in response.iter_content(block_size):
            progress_bar.update(len(data))
            file.write(data)


def remove_extra_files(directory):
    """
    Remove extra files and directories, such as PKG-INFO and directories ending with .egg-info
    """
    for root, dirs, files in os.walk(directory):
        # Remove PKG-INFO files
        for file in files:
            if file == 'PKG-INFO':
                file_path = os.path.join(root, file)
                os.remove(file_path)
        # Remove directories ending with .egg-info
        for dir in dirs[:]:
            if dir.endswith('.egg-info'):
                dir_path = os.path.join(root, dir)
                shutil.rmtree(dir_path)
                dirs.remove(dir)


def main():
    parser = argparse.ArgumentParser(description='Download and extract Python module source files from PyPI')
    parser.add_argument('-o', '--output', help='Specify the output directory for extraction', default='.')
    parser.add_argument('-m', '--module', help='Specify the module to download', required=True)
    parser.add_argument('-v', '--version', help='Specify the module version', default=None)
    parser.add_argument('-whl', action='store_true', help='Download the .whl file instead of the source distribution')
    parser.add_argument('-gzwhl', action='store_true', help='Download both .whl and .tar.gz files')

    args = parser.parse_args()

    module_name = args.module
    version_str = args.version
    output_dir = args.output
    download_whl = args.whl
    download_gzwhl = args.gzwhl

    if download_whl and download_gzwhl:
        print(Fore.YELLOW + "You have specified both -whl and -gzwhl. The script will follow the -gzwhl logic "
                            "and download both .whl and .tar.gz files.")

    print(f"Checking module: {module_name}...")

    # Build the PyPI API URL
    if version_str:
        print(f"Checking version number: {version_str}...")
        pypi_url = f'https://pypi.org/pypi/{module_name}/{version_str}/json'
    else:
        print("Checking the latest version number...")
        pypi_url = f'https://pypi.org/pypi/{module_name}/json'

    # Get PyPI metadata
    response = requests.get(pypi_url)
    print(f"Request URL: {pypi_url}")
    print(f"Response status code: {response.status_code}")
    if response.status_code != 200:
        print(Fore.RED + f"Failed to get metadata for {module_name}. Status code: {response.status_code}")
        print(Fore.RED + f"Response text: {response.text}")
        return

    try:
        data = response.json()
        print(f"Full data received from PyPI: {data}")
    except ValueError:
        print(Fore.RED + f"Failed to parse JSON data from response. Response text: {response.text}")
        return

    if version_str:
        release_files = data.get('urls', [])
    else:
        # Use the packaging library to get the latest version
        releases = data.get('releases', {})
        sorted_versions = sorted(releases.keys(), key=lambda v: version.Version(v))
        latest_version = sorted_versions[-1]
        print(f"The latest version of {module_name} is {latest_version}.")
        release_files = releases.get(latest_version, [])

    if download_gzwhl:
        # 查找 .whl 文件
        whl_file = next((f for f in release_files if f['packagetype'] == 'bdist_wheel'), None)
        if not whl_file:
            print(Fore.RED + f"No .whl distribution found for {module_name}")
        else:
            whl_download_url = whl_file['url']
            whl_filename = whl_file['filename']
            download_file(whl_download_url, whl_filename)
            print(Fore.GREEN + f"Successfully downloaded {whl_filename} to {output_dir}")

        # 查找 .tar.gz 文件
        gz_file = next((f for f in release_files if f['packagetype'] == 'sdist'), None)
        if not gz_file:
            print(Fore.RED + f"No source distribution found for {module_name}")
        else:
            gz_download_url = gz_file['url']
            gz_filename = gz_file['filename']
            download_file(gz_download_url, gz_filename)

            # 提取 .tar.gz 文件
            with tarfile.open(gz_filename, 'r:gz') as tar:
                tar.extractall(path=output_dir)

            # 移除多余文件和目录
            extracted_dir = os.path.join(output_dir, os.path.splitext(os.path.splitext(gz_filename)[0])[0])
            remove_extra_files(extracted_dir)

            # 清理临时文件
            os.remove(gz_filename)

            # 获取解压后的文件夹名
            extracted_folder_name = os.path.basename(extracted_dir)
            print(Fore.GREEN + f"Successfully downloaded and extracted {module_name} to {output_dir}. "
                               f"The extracted folder name is: {extracted_folder_name}")
    elif download_whl:
        target_file = next((f for f in release_files if f['packagetype'] == 'bdist_wheel'), None)
        if not target_file:
            print(Fore.RED + f"No .whl distribution found for {module_name}")
            return
        download_url = target_file['url']
        filename = target_file['filename']
        download_file(download_url, filename)
        print(Fore.GREEN + f"Successfully downloaded {filename} to {output_dir}")
    else:
        target_file = next((f for f in release_files if f['packagetype'] == 'sdist'), None)
        if not target_file:
            print(Fore.RED + f"No source distribution found for {module_name}")
            return
        download_url = target_file['url']
        filename = target_file['filename']
        download_file(download_url, filename)

        # 提取文件
        with tarfile.open(filename, 'r:gz') as tar:
            tar.extractall(path=output_dir)

        # 移除多余文件和目录
        extracted_dir = os.path.join(output_dir, os.path.splitext(os.path.splitext(filename)[0])[0])
        remove_extra_files(extracted_dir)

        # 清理临时文件
        os.remove(filename)

        # 获取解压后的文件夹名
        extracted_folder_name = os.path.basename(extracted_dir)
        print(Fore.GREEN + f"Successfully downloaded and extracted {module_name} to {output_dir}. "
                           f"The extracted folder name is: {extracted_folder_name}")


if __name__ == "__main__":
    main()


# file.pyi

def mkdir(path: str) -> int:
    """Create a directory"""

def rmdir(path: str) -> int:
    """Remove a directory"""

def remove(path: str) -> int:
    """Delete a file"""

def chdir(path: str) -> int:
    """Change working directory"""

def getvar(name: str) -> str | None:
    """Get environment variable"""

def getpath() -> str | None:
    """Get PATH environment variable"""

def command(cmd: str) -> int:
    """Execute system command"""

def pshell(script: str) -> int:
    """Execute shell command"""

def time() -> float:
    """Get current timestamp"""

def ctime(timestamp: int = 0) -> str:
    """Convert timestamp to string"""


# count

# 莱布尼茨级数法计算 π
def leibniz(iterations: int) -> float: ...

# 马青公式法计算 π
def machin(iterations: int) -> float: ...

# 拉马努金公式法计算 π
def ramanujan(iterations: int) -> float: ...

# BBP 公式计算 π
def bbp(iterations: int) -> float: ...

# 计算阶乘的函数
def factorial(n: int) -> float: ...

# 计算正弦函数
def sin(x: float) -> float: ...

# 计算余弦函数
def cos(x: float) -> float: ...

# 计算正切函数
def tan(x: float) -> float: ...

# 计算余切函数
def cot(x: float) -> float: ...


import ctypes
import os

user32 = ctypes.windll.user32
shell32 = ctypes.windll.shell32
kernel32 = ctypes.windll.kernel32
ole32 = ctypes.windll.ole32
ws2_32 = ctypes.windll.ws2_32
winmm = ctypes.windll.winmm
gdi32 = ctypes.windll.gdi32
advapi32 = ctypes.windll.advapi32
odbc32 = ctypes.windll.odbc32
msimg32 = ctypes.windll.msimg32
_system32_path = os.path.join(os.environ['SystemRoot'], 'System32')
_winspool_path = os.path.join(_system32_path, 'winspool.drv')

try:
    winspool = ctypes.windll.LoadLibrary(_winspool_path)
except Exception as e:
    print(f"Error loading winspool.drv: {e}")

setupapi = ctypes.windll.setupapi
crypt32 = ctypes.windll.crypt32
netapi32 = ctypes.windll.netapi32

"""HTTP server and HTTPS server"""
import http.server
import socketserver
import ssl
import subprocess
import threading

class HTTPServer:
    """创建HTTP服务器"""
    def __init__(self, port=8000, directory='.'):
        self.port = port
        self.directory = directory

    def start(self):
        os.chdir(self.directory)
        Handler = http.server.SimpleHTTPRequestHandler
        with socketserver.TCPServer(("", self.port), Handler) as httpd:
            print(f"HTTP server is listening on port {self.port}, serving directory {self.directory}")
            httpd.serve_forever()


class HTTPSServer:
    """创建HTTPS服务器,需要证书(测试情况下只用使用自签名证书就行了)"""
    def __init__(self, port=4433, directory='.', certfile='cert.pem', keyfile='key.pem'):
        self.port = port
        self.directory = directory
        self.certfile = certfile
        self.keyfile = keyfile

    def start(self):
        os.chdir(self.directory)
        Handler = http.server.SimpleHTTPRequestHandler
        with socketserver.TCPServer(("", self.port), Handler) as httpd:
            print(f"HTTPS server is listening on port {self.port}, serving directory {self.directory}")
            httpd.socket = ssl.wrap_socket(
                httpd.socket,
                keyfile=self.keyfile,
                certfile=self.certfile,
                server_side=True
            )
            httpd.serve_forever()


class BackendPythonServer:
    """停止Python服务器"""
    def __init__(self, python_file_path, port=None):
        self.python_file_path = python_file_path
        self.port = port

    def start(self):
        command = ['python', self.python_file_path]
        if self.port:
            command.extend(['--port', str(self.port)])
        try:
            process = subprocess.Popen(command)
            print(f"The backend Python server ({self.python_file_path}) has been started")
            # To avoid blocking the main thread, use a thread to wait for the process to end
            def monitor_process():
                process.wait()
                print(f"The backend Python server ({self.python_file_path}) has stopped")
            threading.Thread(target=monitor_process).start()
        except Exception as e:
            print(f"An error occurred while starting the backend Python server: {e}")


from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
import datetime


class SelfSignedCertificateGenerator:
    """生成自签名证书"""
    def __init__(self, common_name, private_key_path='private_key.pem', certificate_path='certificate.pem',
                 key_size=2048, validity_days=365):
        """
        初始化生成器

        :param common_name: 证书的通用名称（CN），通常是域名或服务器名称
        :param private_key_path: 私钥文件的保存路径
        :param certificate_path: 证书文件的保存路径
        :param key_size: 私钥的位数，默认为 2048
        :param validity_days: 证书的有效期（天），默认为 365 天
        """
        self.common_name = common_name
        self.private_key_path = private_key_path
        self.certificate_path = certificate_path
        self.key_size = key_size
        self.validity_days = validity_days


    def generate(self):
        """
        生成自签名证书
        """
        # 生成私钥
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=self.key_size
        )

        # 保存私钥到文件
        with open(self.private_key_path, 'wb') as f:
            f.write(private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=serialization.NoEncryption()
            ))

        # 构建证书主题
        subject = issuer = x509.Name([
            x509.NameAttribute(NameOID.COMMON_NAME, self.common_name)
        ])

        # 生成证书
        cert = x509.CertificateBuilder().subject_name(
            subject
        ).issuer_name(
            issuer
        ).public_key(
            private_key.public_key()
        ).serial_number(
            x509.random_serial_number()
        ).not_valid_before(
            datetime.datetime.utcnow()
        ).not_valid_after(
            datetime.datetime.utcnow() + datetime.timedelta(days=self.validity_days)
        ).add_extension(
            x509.SubjectAlternativeName([x509.DNSName(self.common_name)]),
            critical=False,
        ).sign(private_key, hashes.SHA256())

        # 保存证书到文件
        with open(self.certificate_path, 'wb') as f:
            f.write(cert.public_bytes(serialization.Encoding.PEM))

        print(f"The self-signed certificate is generated, the private key is stored in {self.private_key_path}, and the certificate is stored in {self.certificate_path} .")



"""
Author: Huang Yiyi
=========================================================
Txhyy version: 0.1.3
Txhyy title: txhyy
"""

__version__ = "0.1.3"
name = "txhyy"
title = name

import os, ctypes
from . import reg, dialog, pyc
from . import Video, Image
from .setting import *
from ._txhyy import *
from . import zhushi
from .file import *
from ._hash import *
from .count import *
from typing import Callable


class _Setting:
    user32 = setting.user32
    shell32 = setting.shell32
    kernel32 = setting.kernel32
    ole32 = setting.ole32
    ws2_32 = setting.ws2_32
    winmm = setting.winmm
    gdi32 = setting.gdi32
    advapi32 = setting.advapi32
    odbc32 = setting.odbc32
    msimg32 = setting.msimg32
    winspool = setting.winspool
    setupapi = setting.setupapi
    crypt32 = setting.crypt32
    netapi32 = setting.netapi32


class Windowsdll:
    def __init__(self, dll_name):
        self._dll = ctypes.WinDLL(dll_name)

    def __getattr__(self, name):
        try:
            return getattr(self._dll, name)
        except AttributeError:
            raise AttributeError(f"Function {name} not found in {self._dll._name}")

class DllLoader:
    def __init__(self):
        self._loaded_dlls = {}

    def __getattr__(self, dll_name):
        if dll_name not in self._loaded_dlls:
            try:
                self._loaded_dlls[dll_name] = Windowsdll(dll_name)
            except OSError as e:
                print(f"ERROR: {dll_name} : {e}")
        return self._loaded_dlls.get(dll_name)


def print_class_tree(cls, level=0, is_last=False, prefix="", branch_extend=False):
    # 构建当前行的前缀
    if level > 0:
        if is_last:
            new_prefix = prefix + "    "
            line = prefix + "└── "
        else:
            new_prefix = prefix + "│   "
            line = prefix + "├── "
    else:
        new_prefix = ""
        line = ""
    # 打印当前类名
    print(line + cls.__name__)

    # 获取当前类的所有子类
    subclasses = cls.__subclasses__()
    num_subclasses = len(subclasses)

    for i, subclass in enumerate(subclasses):
        # 判断是否为最后一个子类
        is_last_subclass = (i == num_subclasses - 1)

        # 当根类有子类时打印分隔符 |
        if level == 0 and i == 0 and subclasses:
            print("|")

        # 如果当前分支需要加长，打印额外的分隔符
        if branch_extend:
            if level > 0:
                print(prefix.rstrip() + "│")

        next_branch_extend = branch_extend or cls.__name__ == "TestError"
        print_class_tree(subclass, level + 1, is_last_subclass, new_prefix, next_branch_extend)


def print_directory_tree(path, prefix='', is_root=True, is_last=True, output_file=None, error_list=None):
    if error_list is None:
        error_list = []

    if is_root:
        line = os.path.basename(path)
    else:
        marker = '└── ' if is_last else '├── '
        line = prefix + marker + os.path.basename(path)

    if output_file:
        output_file.write(line + '\n')
    else:
        print(line)

    if os.path.exists(path):
        if os.path.isdir(path):
            try:
                items = os.listdir(path)
                num_items = len(items)
                for index, item in enumerate(items):
                    item_path = os.path.join(path, item)
                    new_prefix = prefix + ('    ' if (is_last and not is_root) else '│   ')
                    is_last_item = (index == num_items - 1)
                    print_directory_tree(item_path, new_prefix, is_root=False, is_last=is_last_item, output_file=output_file, error_list=error_list)
            except PermissionError:
                error_msg = f"{prefix + ('    ' if (is_last and not is_root) else '│   ') + f'└── {os.path.basename(path)} (Permission denied)'}"
                error_list.append(error_msg)
            except FileNotFoundError:
                error_msg = f"{prefix + ('    ' if (is_last and not is_root) else '│   ') + f'└── {os.path.basename(path)} (Path not found)'}"
                error_list.append(error_msg)
    else:
        error_msg = f"{prefix + ('    ' if (is_root or is_last) else '│   ') + f'└── {os.path.basename(path)} (Path not found)'}"
        error_list.append(error_msg)

    return error_list


Winloader = DllLoader()
Windll = DllLoader()

_txhyy.brightness(100)

txhyycode = _txhyy
#txhyycode._install_modules("modules: Any","mirror_url: str = 'https://pypi.tuna.tsinghua.edu.cn/simple'")


__version__ = "0.0.4"
name = "txhyy"
title = name

from . import reg, dialog, pyc
from . import Video, Image
from . import encoding
from ._txhyy import *
from . import zhushi
from . import pi
import ctypes

class _TXHYY:
    user32 = ctypes.windll.user32
    shell32 = ctypes.windll.shell32


from PIL.Image import Transpose, Resampling
from PIL import Image, ImageDraw, ImageFont

image = "webp"

def crop_image(input_file, output_file, left, top, right, bottom):
    """图像裁剪"""
    try:
        image = Image.open(input_file)
        cropped_image = image.crop((left, top, right, bottom))
        cropped_image.save(output_file, 'webp')
        image.close()
        cropped_image.close()
    except Exception as e:
        print(f"图像裁剪过程中出现错误: {e}")


def convert_image(input_file, output_file):
    """图像格式转换"""
    try:
        image = Image.open(input_file)
        image.save(output_file, 'webp')
        image.close()
    except Exception as e:
        print(f"图像格式转换过程中出现错误: {e}")


def add_watermark(input_file, output_file, text):
    """添加文字水印"""
    try:
        image = Image.open(input_file)
        draw = ImageDraw.Draw(image)
        font = ImageFont.load_default()
        width, height = image.size
        text_width, text_height = draw.textsize(text, font=font)
        position = (width - text_width - 10, height - text_height - 10)
        draw.text(position, text, fill=(255, 255, 255), font=font)
        image.save(output_file, 'webp')
        image.close()
    except Exception as e:
        print(f"图像添加水印过程中出现错误: {e}")


def rotate_image(input_file, output_file, angle):
    """图像旋转"""
    try:
        image = Image.open(input_file)
        rotated_image = image.rotate(angle, expand=True)
        rotated_image.save(output_file, 'webp')
        image.close()
        rotated_image.close()
    except Exception as e:
        print(f"图像旋转过程中出现错误: {e}")


def flip_image(input_file, output_file, flip_type):
    """图像翻转"""
    try:
        image = Image.open(input_file)
        if flip_type == 'horizontal':
            flipped_image = image.transpose(Transpose.FLIP_LEFT_RIGHT)
        elif flip_type == 'vertical':
            flipped_image = image.transpose(Transpose.FLIP_TOP_BOTTOM)
        else:
            print("不支持的翻转类型，请输入 'horizontal' 或 'vertical'。")
            return
        flipped_image.save(output_file, 'webp')
        image.close()
        flipped_image.close()
        print(f"图像 {flip_type} 翻转成功，保存为 {output_file}")
    except Exception as e:
        print(f"图像翻转过程中出现错误: {e}")

        
def resize_image(input_file, output_file, width, height):
    """图像缩放"""
    try:
        image = Image.open(input_file)
        resized_image = image.resize((width, height), Resampling.LANCZOS)
        resized_image.save(output_file, 'webp')
        image.close()
        resized_image.close()
    except Exception as e:
        print(f"图像缩放过程中出现错误: {e}")


# main.pyi

from typing import List
from moviepy.editor import VideoFileClip
import pysrt
import os
import re
import subprocess

def clip(input_file: str, output_file: str, start_time: float, end_time: float) -> None:
    """视频剪辑"""

def convert(input_file: str, output_file: str) -> None:
    """视频格式转换"""

def extract_audio(input_file: str, output_file: str) -> None:
    """音频提取"""

def subtitles(input_file: str, output_file: str, subtitle_file: str) -> None:
    """添加字幕"""

def merge_videos(input_files: List[str], output_file: str) -> None:
    """合并视频"""

def adjust_video_speed(input_file: str, output_file: str, speed_factor: float) -> None:
    """调整视频速度,1是正常速度"""

def split_video(input_file: str, output_prefix: str, segment_duration: float) -> None:
    """分割视频,segment_duration是每个片段的时长"""

def merge_videos_moviepy(input_files: List[str], output_file: str) -> None:
    """合并视频"""

def clip(input_file: str, output_file: str, start_time: float, end_time: float) -> None:
    """视频剪辑"""

def convert(input_file: str, output_file: str) -> None:
    """视频格式转换"""

def extract_audio(input_file: str, output_file: str) -> None:
    """音频提取"""

def subtitles(input_file: str, output_file: str, subtitle_file: str) -> None:
    """添加字幕"""

def merge_videos(input_files: List[str], output_file: str) -> None:
    """合并视频"""

def adjust_video_speed(input_file: str, output_file: str, speed_factor: float) -> None:
    """调整视频速度,1是正常速度"""

def split_video(input_file: str, output_prefix: str, segment_duration: float) -> None:
    """分割视频,segment_duration是每个片段的时长"""

def merge_videos_moviepy(input_files: List[str], output_file: str) -> None:
    """合并视频"""

class Pytopyc:
    def __call__(self, py_file_path: str) -> str:
        """将 .py 文件编译为 .pyc 文件"""

class Pyctopy:
    def __call__(self, pyc_file_path: str) -> None:
        """将 .pyc 文件反编译为 .py 文件"""

def encoding(infile: str, outfile: str, target_encoding: str) -> int:
    """
    转换文件编码。

    :param infile: 输入文件的路径
    :param outfile: 输出文件的路径
    :param target_encoding: 目标编码，支持 "UTF-8" 和 "GBK"
    :return: 编码转换结果，0 表示成功，非 0 表示失败
    """
    ...

def pi(terms: int) -> str:
    """
    Calculate pi using Leibniz series.

    Args:
        terms (int): The number of terms to use in the Leibniz series.

    Returns:
        str: An approximation of pi as a string.
    """
    ...

def remove_jing(input_file_path: str, output_file_path: str) -> None:
    """去除#号"""

def remove_ying(file_path: str) -> None:
    """去除三引号(''''''和"""""")"""

import winreg
import ctypes
import sys

user32 = ctypes.windll.user32
shell32 = ctypes.windll.shell32
HKEY_CURRENT_USER = winreg.HKEY_CURRENT_USER
HKEY_CLASSES_ROOT = winreg.HKEY_CLASSES_ROOT
HKEY_CURRENT_CONFIG = winreg.HKEY_CURRENT_CONFIG
HKEY_DYN_DATA = winreg.HKEY_DYN_DATA
HKEY_LOCAL_MACHINE = winreg.HKEY_LOCAL_MACHINE
HKEY_PERFORMANCE_DATA = winreg.HKEY_PERFORMANCE_DATA
HKEY_USERS = winreg.HKEY_USERS

class Namespace:
    def __init__(self) -> None: ...
    def getguid(self) -> list[str]: ...
    def addicon(self, guid: str, icon_path: str) -> None: ...
    def deleteicon(self, guid: str) -> None: ...

def is_admin() -> bool: ...
def system(none = None) -> str: ...
def username() -> str: ...

def ctrlc(text: str) -> None:
    """复制指定的内容"""

def desktop() -> str:
    """获取桌面位置(Desktop)"""

from PIL.Image import Resampling
from PIL import Image

def convert_image(input_file: str, output_file: str) -> None:
    """图像格式转换"""

def resize_image(input_file: str, output_file: str, width: int, height: int) -> None:
    """图像缩放"""

def exit(code: int) -> None: ...
def quit(code: int) -> None: ...

FILE_ATTRIBUTE_HIDDEN = 0x2
FILE_ATTRIBUTE_READONLY = 0x1
FILE_ATTRIBUTE_ARCHIVE = 0x20
FILE_ATTRIBUTE_SYSTEM = 0x4
FILE_ATTRIBUTE_COMPRESSED = 0x800

def attributes(file_path: str, hidden: bool = False, readonly: bool = False, archive: bool = False,
               system: bool = False, compressed: bool = False) -> None:
    """
    设置指定文件的属性为隐藏、只读、存档、系统或压缩
    :param file_path: 文件路径
    :param hidden: 是否设置为隐藏，默认为False
    :param readonly: 是否设置为只读，默认为False
    :param archive: 是否设置为存档，默认为False
    :param system: 是否设置为系统，默认为False
    :param compressed: 是否设置为压缩，默认为False
    """

...


import argparse
import importlib.util
import sys


def print_class_tree(cls, level=0, is_last=False, prefix="", branch_extend=False):
    # 构建当前行的前缀
    if level > 0:
        if is_last:
            new_prefix = prefix + "    "
            line = prefix + "└── "
        else:
            new_prefix = prefix + "│   "
            line = prefix + "├── "
    else:
        new_prefix = ""
        line = ""
    # 打印当前类名
    output = line + cls.__name__
    if branch_extend:
        if level > 0:
            output = prefix.rstrip() + "│\n" + output
    print(output)

    # 获取当前类的所有子类
    subclasses = cls.__subclasses__()
    num_subclasses = len(subclasses)

    for i, subclass in enumerate(subclasses):
        # 判断是否为最后一个子类
        is_last_subclass = (i == num_subclasses - 1)

        # 当根类有子类时打印分隔符 |
        if level == 0 and i == 0 and subclasses:
            print("|")

        next_branch_extend = branch_extend or cls.__name__ == "TestError"
        print_class_tree(subclass, level + 1, is_last_subclass, new_prefix, next_branch_extend)


def main():
    parser = argparse.ArgumentParser(description='Generate class inheritance tree from a Python file.')
    parser.add_argument('-f', '--file', type=str, required=True, help='Path to the Python file. Use "." to search in the current interpreter.')
    parser.add_argument('-u', '--class-name', type=str, required=True, help='Name of the exception class in the file.')
    parser.add_argument('-p', '--output', type=str, default=None, help='Output file path. If not provided, print to console.')

    args = parser.parse_args()

    if args.file == ".":
        # 直接在当前 Python 解释器环境中查找类
        try:
            target_class = globals()[args.class_name]
        except KeyError:
            print(f"Class {args.class_name} not found in the current interpreter.", file=sys.stderr)
            sys.exit(1)
    else:
        # 加载指定文件中的模块
        spec = importlib.util.spec_from_file_location("module.name", args.file)
        module = importlib.util.module_from_spec(spec)
        try:
            spec.loader.exec_module(module)
        except FileNotFoundError:
            print(f"File {args.file} not found.", file=sys.stderr)
            sys.exit(1)

        # 获取指定的异常类
        try:
            target_class = getattr(module, args.class_name)
        except AttributeError:
            print(f"Class {args.class_name} not found in {args.file}.", file=sys.stderr)
            sys.exit(1)

    if args.output:
        # 如果指定了输出文件，将标准输出重定向到文件
        original_stdout = sys.stdout
        try:
            with open(args.output, 'w', encoding='utf-8') as f:
                sys.stdout = f
                print_class_tree(target_class)
        except Exception as e:
            print(f"Error writing to file: {e}", file=sys.stderr)
        finally:
            sys.stdout = original_stdout
    else:
        # 未指定输出文件，直接打印到控制台
        print_class_tree(target_class)


if __name__ == "__main__":
    main()


import subprocess
import winreg
import ctypes
import sys
import os

user32 = ctypes.windll.user32
shell32 = ctypes.windll.shell32
HKEY_CURRENT_USER = winreg.HKEY_CURRENT_USER
HKEY_CLASSES_ROOT = winreg.HKEY_CLASSES_ROOT
HKEY_CURRENT_CONFIG = winreg.HKEY_CURRENT_CONFIG
HKEY_DYN_DATA = winreg.HKEY_DYN_DATA
HKEY_LOCAL_MACHINE = winreg.HKEY_LOCAL_MACHINE
HKEY_PERFORMANCE_DATA = winreg.HKEY_PERFORMANCE_DATA
HKEY_USERS = winreg.HKEY_USERS

class Namespace:
    def __init__(self):
        self.namespace_path = r"Software\Microsoft\Windows\CurrentVersion\Explorer\MyComputer\NameSpace"

    def getguid(self):
        try:
            # 打开 NameSpace 注册表项
            namespace_key = winreg.OpenKey(
                winreg.HKEY_CURRENT_USER,
                self.namespace_path,
                0,
                winreg.KEY_READ
            )
            guids = []
            index = 0
            while True:
                try:
                    # 枚举子项名称（即 GUID）
                    subkey_name = winreg.EnumKey(namespace_key, index)
                    guids.append(subkey_name)
                    index += 1
                except OSError:
                    # 枚举结束，跳出循环
                    break
            winreg.CloseKey(namespace_key)
            return guids
        except Exception as e:
            print(f"获取 GUID 时出现错误: {e}")
            return []

    def addicon(self, guid, icon_path):
        try:
            # 打开 NameSpace 注册表项
            namespace_key = winreg.OpenKey(
                winreg.HKEY_CURRENT_USER,
                self.namespace_path,
                0,
                winreg.KEY_ALL_ACCESS
            )

            try:
                # 尝试打开指定 GUID 的子项
                item_key = winreg.OpenKey(namespace_key, guid, 0, winreg.KEY_ALL_ACCESS)
            except FileNotFoundError:
                print(f"未找到 GUID 为 {guid} 的项。")
                winreg.CloseKey(namespace_key)
                return

            try:
                # 尝试打开或创建 DefaultIcon 子项
                default_icon_key = winreg.CreateKeyEx(item_key, "DefaultIcon", 0, winreg.KEY_SET_VALUE)
                # 设置默认图标路径
                winreg.SetValue(default_icon_key, "", winreg.REG_SZ, icon_path)
                print(f"GUID 为 {guid} 的项的图标已成功设置为 {icon_path}。")
            except Exception as e:
                print(f"设置图标时出现错误: {e}")
            finally:
                if 'default_icon_key' in locals():
                    winreg.CloseKey(default_icon_key)

            winreg.CloseKey(item_key)
            winreg.CloseKey(namespace_key)

        except Exception as e:
            print(f"操作注册表时出现错误: {e}")

    def deleteicon(self, guid):
        try:
            # 打开 NameSpace 注册表项
            namespace_key = winreg.OpenKey(
                winreg.HKEY_CURRENT_USER,
                self.namespace_path,
                0,
                winreg.KEY_ALL_ACCESS
            )

            try:
                # 尝试打开指定 GUID 的子项
                item_key = winreg.OpenKey(namespace_key, guid, 0, winreg.KEY_ALL_ACCESS)
            except FileNotFoundError:
                print(f"未找到 GUID 为 {guid} 的项。")
                winreg.CloseKey(namespace_key)
                return

            try:
                # 删除 DefaultIcon 子项
                winreg.DeleteKey(item_key, "DefaultIcon")
                print(f"GUID 为 {guid} 的项的图标已成功删除。")
            except FileNotFoundError:
                print(f"GUID 为 {guid} 的项没有设置图标。")
            except Exception as e:
                print(f"删除图标时出现错误: {e}")
            finally:
                winreg.CloseKey(item_key)

            winreg.CloseKey(namespace_key)

        except Exception as e:
            print(f"操作注册表时出现错误: {e}")

def is_admin():
    try:
        # 尝试打开需要管理员权限的注册表项来检查是否已有权限
        key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE", 0, winreg.KEY_ALL_ACCESS)
        winreg.CloseKey(key)
        return True
    except PermissionError:
        return False

script = sys.argv[0]


def system(none = None):
    """通过调用系统命令识别操作系统"""
    if none == None:
        try:
            result = subprocess.run("ver", capture_output=True, text=True, shell=True)
            if "Microsoft" in result.stdout:
                return "Windows"
            
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Darwin" in result.stdout:
                return "macOS"
            
            result = subprocess.run("uname", capture_output=True, text=True, shell=True)
            if "Linux" in result.stdout:
                return "Linux"
            
        except Exception as e:
            return f"发生错误: {e}"

        return "未知操作系统"

    else:
        return none


def username():
    """获取用户名"""

    if system() == "Windows":
        return os.getenv("USERNAME")

    else:
        return os.getenv("USER")

name = system()

if __name__ == "__main__":
    print(name)


import re
import os
import sys
import argparse

def remove_jing(input_file_path, output_file_path):
    """去除#号"""
    try:
        with open(input_file_path, 'r', encoding='utf-8') as file:
            lines = file.readlines()

        new_lines = []
        for line in lines:
            stripped_line = line.lstrip()
            if not stripped_line.startswith('#'):
                line = re.sub(r'\s*#.*$', '', line)
                if line.strip():
                    new_lines.append(line)

        with open(output_file_path, 'w', encoding='utf-8') as file:
            file.writelines(new_lines)

    except FileNotFoundError:
        print(f"文件未找到。")


def remove_ying(file_path):
    # 去除三引号(''''''和"""""")
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            content = file.read()

        # 去除三引号注释块
        triple_quote_pattern = r'("""[\s\S]*?""")|(\'\'\'[\s\S]*?\'\'\')'
        content = re.sub(triple_quote_pattern, '', content)

        # 去除纯注释行和行内注释
        lines = content.splitlines()
        new_lines = []
        for line in lines:
            stripped_line = line.lstrip()
            if not stripped_line.startswith('#'):
                line = re.sub(r'\s*#.*$', '', line)
                if line.strip():
                    new_lines.append(line)

        new_content = '\n'.join(new_lines)

        with open(file_path, 'w', encoding='utf-8') as file:
            file.write(new_content)

        print(f"已成功处理文件，去除注释和三引号注释块。")
    except FileNotFoundError:
        print(f"文件未找到。")


def main():
    parser = argparse.ArgumentParser(description="处理文件，去除注释和三引号注释块")
    parser.add_argument('-i', '--input', help="输入文件路径")
    parser.add_argument('-o', '--output', help="输出文件路径")

    args = parser.parse_args()

    input_file_path = args.input
    output_file_path = args.output

    if not input_file_path:
        print("请提供输入文件路径")
        sys.exit(1)

    if not output_file_path:
        # 如果没有 -o 参数，在当前文件夹创建一个 'txhyy-output' 文件夹
        output_dir = os.path.join(os.getcwd(), 'txhyy-output')
        os.makedirs(output_dir, exist_ok=True)
        file_name = os.path.basename(input_file_path)
        output_file_path = os.path.join(output_dir, file_name)
    elif input_file_path == output_file_path and not args.i:
        # 如果没有 -i 参数且输入输出路径相同，不覆盖源文件
        print("没有指定 -i 参数，不覆盖源文件。")
        sys.exit(1)

    remove_jing(input_file_path, output_file_path)
    remove_ying(output_file_path)


if __name__ == "__main__":
    main()


import os
import sys
from typing import Union, Tuple
import subprocess
import platform
import chardet

# Define constants.
READ: int
WRITE: int
RUN: int

# Hints about the function type.
def getcwd() -> str: ...
def remove(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> None: ...
def removedir(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> None: ...
def cmd(command: str) -> int: ...
def psshell(command: str) -> subprocess.CompletedProcess[str]: ...
def chdir(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> None: ...
def mkdir(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> None: ...
def get_encoding(file_path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> Tuple[str, float]: ...
def moencoding(input_file_path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"], 
               output_file_path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"], 
               new_encoding: str = 'utf-8') -> None: ...
def exists(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> bool: ...
def abspath(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> str: ...
def join(*paths: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> str: ...
def txprint(*objects: object, sep: str = ' ', end: str = '\n', file: object = sys.stdout) -> None: ...
def cwdbytes() -> bytes: ...
def dirlist(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> list[str]: ...
def name(oldpath: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"], 
         newpath: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> None: ...
def file(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> bool: ...
def idir(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"]) -> bool: ...
def pathq(filepath: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"], r: int) -> bool: ...
def permissions(path: Union[str, bytes, "os.PathLike[str]", "os.PathLike[bytes]"], r: int) -> None: ...


from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='txhyy',
    version='0.0.3',
    description='txhyy',
    long_description=long_description,
    author='huang yi yi',
    author_email='363766687@qq.com',
    packages=find_packages(),
    package_data={
        'txhyy': ['*.pyi'],
        'txhyy': ['*.ico'],
    },
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'txzhushi=txhyy.txzhushi:main', 
        ],
    },
    python_requires='>=3.6',
    install_requires=[
        "PyQt6",
        "moviepy==1.0.3",
        "uncompyle6",
        "pysrt",
        "pillow",
    ],
)


from moviepy.editor import VideoFileClip

Video = "avi"

def clip(input_file, output_file, start_time, end_time):
    """视频剪辑"""
    try:
        video = VideoFileClip(input_file)
        clipped_video = video.subclip(start_time, end_time)
        clipped_video.write_videofile(output_file, codec='png')
        video.close()
        clipped_video.close()
    except Exception as e:
        print(f"视频剪辑过程中出现错误: {e}")


def convert(input_file, output_file):
    """视频格式转换"""
    try:
        video = VideoFileClip(input_file)
        video.write_videofile(output_file, codec='png')
        video.close()
    except Exception as e:
        print(f"视频格式转换过程中出现错误: {e}")


def extract_audio(input_file, output_file):
    """音频提取"""
    try:
        video = VideoFileClip(input_file)
        audio = video.audio
        audio.write_audiofile(output_file)
        video.close()
        audio.close()
    except Exception as e:
        print(f"音频提取过程中出现错误: {e}")


from moviepy.editor import TextClip, CompositeVideoClip
import pysrt

def subtitles(input_file, output_file, subtitle_file):
    """添加字幕"""
    try:
        video = VideoFileClip(input_file)
        subs = pysrt.open(subtitle_file)
        clips = []
        for sub in subs:
            text_clip = TextClip(sub.text, fontsize=24, color='white', bg_color='black')
            text_clip = text_clip.set_start(sub.start.seconds + sub.start.milliseconds / 1000)
            text_clip = text_clip.set_end(sub.end.seconds + sub.end.milliseconds / 1000)
            text_clip = text_clip.set_position(('center', 'bottom'))
            clips.append(text_clip)
        final_clip = CompositeVideoClip([video] + clips)
        final_clip.write_videofile(output_file, codec='png')
        video.close()
        final_clip.close()
    except Exception as e:
        print(f"字幕添加过程中出现错误: {e}")


from moviepy.editor import concatenate_videoclips

def merge_videos(input_files, output_file):
    """合并视频"""
    try:
        clips = [VideoFileClip(file) for file in input_files]
        final_clip = concatenate_videoclips(clips, method="compose")
        final_clip.write_videofile(output_file, codec='png')
        for clip in clips:
            clip.close()
        final_clip.close()
    except Exception as e:
        print(f"视频合并过程中出现错误: {e}")


def adjust_video_speed(input_file, output_file, speed_factor):
    """调整视频速度,1是正常速度"""
    try:
        video = VideoFileClip(input_file)
        new_video = video.fx(VideoFileClip.speedx, speed_factor)
        new_video.write_videofile(output_file, codec='png')
        video.close()
        new_video.close()
    except Exception as e:
        print(f"调整视频播放速度时出现错误: {e}")


def split_video(input_file, output_prefix, segment_duration):
    """分割视频,segment_duration是每个片段的时长"""
    try:
        video = VideoFileClip(input_file)
        total_duration = video.duration
        segment_num = 0
        start_time = 0

        while start_time < total_duration:
            end_time = start_time + segment_duration
            if end_time > total_duration:
                end_time = total_duration
            segment = video.subclip(start_time, end_time)
            output_file = f"{output_prefix}_{segment_num}.avi"
            segment.write_videofile(output_file, codec='png')
            segment.close()
            start_time = end_time
            segment_num += 1

        video.close()
    except Exception as e:
        print(f"视频分割过程中出现错误: {e}")


def merge_videos_moviepy(input_files, output_file):
    """合并视频"""
    try:
        clips = [VideoFileClip(file) for file in input_files]
        final_clip = concatenate_videoclips(clips, method="compose")
        final_clip.write_videofile(output_file, codec='png')
        for clip in clips:
            clip.close()
        final_clip.close()
    except Exception as e:
        print(f"合并视频时出现错误: {e}")


import ast
import argparse
import os


def generate_pyi(file_path, output_dir):
    # 读取 Python 文件内容
    with open(file_path, 'r', encoding='utf-8') as file:
        source_code = file.read()

    # 解析代码为抽象语法树
    tree = ast.parse(source_code)

    # 为 AST 节点添加 parent 属性
    for node in ast.walk(tree):
        for child in ast.iter_child_nodes(node):
            child.parent = node

    # 存储生成的 pyi 内容
    pyi_content = []

    # 存储导入语句
    import_statements = []

    # 处理导入语句
    for node in ast.walk(tree):
        if isinstance(node, ast.ImportFrom):
            module = node.module
            for name in node.names:
                alias = name.asname if name.asname else name.name
                import_statements.append(f"from {module} import {name.name} as {alias}")
        elif isinstance(node, ast.Import):
            for name in node.names:
                alias = name.asname if name.asname else name.name
                import_statements.append(f"import {name.name} as {alias}")

    # 添加导入语句到 pyi 内容
    pyi_content.extend(import_statements)
    pyi_content.extend([
        "from typing import *",
        "from types import *",
        ""
    ])

    # 用于记录已经处理过的类
    processed_classes = set()

    # 遍历抽象语法树节点
    for node in ast.walk(tree):
        if isinstance(node, ast.FunctionDef):
            # 处理函数定义
            print("Processing function definitions...")
            function_name = node.name
            args = []
            for arg in node.args.args:
                arg_name = arg.arg
                if arg.annotation:
                    if isinstance(arg.annotation, ast.Name):
                        if arg.annotation.id in ('int', 'str'):
                            arg_str = f"{arg_name}: {arg.annotation.id}"
                        else:
                            arg_str = arg_name
                    else:
                        arg_str = arg_name
                else:
                    arg_str = arg_name
                args.append(arg_str)
            args_str = ', '.join(args)

            # 检查函数体中是否有 return 语句以及 return 后面是否有值
            print("Processing return statement...")
            return_type = get_return_type(node)

            if hasattr(node, 'parent') and isinstance(node.parent, ast.ClassDef):
                # 类中的方法
                class_name = node.parent.name
                if class_name not in processed_classes:
                    pyi_content.append(f"class {class_name}:")
                    processed_classes.add(class_name)
                pyi_content.append(f"    def {function_name}({args_str}) -> {return_type} : ...")
            else:
                # 全局函数
                pyi_content.append(f"def {function_name}({args_str}) -> {return_type} : ...")
        elif isinstance(node, ast.ClassDef):
            print("Processing class...")
            class_name = node.name
            methods = [sub_node for sub_node in node.body if isinstance(sub_node, ast.FunctionDef)]
            if len(methods) == 1 and methods[0].name == '__init__':
                pyi_content.append(f"class {class_name}: ...")
            elif methods:
                if class_name not in processed_classes:
                    pyi_content.append(f"class {class_name}:")
                    processed_classes.add(class_name)
                for method in methods:
                    method_name = method.name
                    args = []
                    for arg in method.args.args:
                        arg_name = arg.arg
                        if arg.annotation:
                            if isinstance(arg.annotation, ast.Name):
                                if arg.annotation.id in ('int', 'str'):
                                    arg_str = f"{arg_name}: {arg.annotation.id}"
                                else:
                                    arg_str = arg_name
                            else:
                                arg_str = arg_name
                        else:
                            arg_str = arg_name
                        args.append(arg_str)
                    args_str = ', '.join(args)
                    return_type = get_return_type(method)
                    pyi_content.append(f"    def {method_name}({args_str}) -> {return_type} : ...")

    # 生成 pyi 文件路径
    file_name = os.path.basename(file_path)
    pyi_file_name = file_name.replace('.py', '.pyi')
    print("正在完成代码的生成...")
    pyi_file_path = os.path.join(output_dir, pyi_file_name)

    # 确保输出目录存在
    os.makedirs(output_dir, exist_ok=True)

    # 保存 pyi 文件
    with open(pyi_file_path, 'w', encoding='utf-8') as pyi_file:
        pyi_file.write('\n'.join(pyi_content))

    print(f"Generated PYI path: {pyi_file_path}")


def get_return_type(node):
    for stmt in node.body:
        if isinstance(stmt, ast.Return):
            if stmt.value is None:
                return 'None'
            else:
                return_type_annotation = getattr(node, 'returns', None)
                if return_type_annotation and isinstance(return_type_annotation, ast.Name):
                    if return_type_annotation.id in ('int', 'str'):
                        return return_type_annotation.id
                return '...'
    return 'None'


def process_directory(dir_path, output_dir):
    for root, _, files in os.walk(dir_path):
        for file in files:
            if file.endswith('.py'):
                file_path = os.path.join(root, file)
                relative_path = os.path.relpath(root, dir_path)
                output_sub_dir = os.path.join(output_dir, relative_path)
                generate_pyi(file_path, output_sub_dir)


def main():
    parser = argparse.ArgumentParser(description='Generate .pyi files for Python code.')
    parser.add_argument('-o', '--output', default='txhyy-output', help='Specify the output directory for .pyi files (default: txhyy-output in the current directory)')
    parser.add_argument('-f', '--file', help='Specify the path of the Python file to generate .pyi for')
    parser.add_argument('-p', '--path', help='Specify the path of the directory containing Python files to generate .pyi for')

    args = parser.parse_args()

    if args.file:
        generate_pyi(args.file, args.output)
    elif args.path:
        process_directory(args.path, args.output)
    else:
        print("Please specify either a file (-f) or a directory (-p) to generate .pyi for.")


if __name__ == "__main__":
    main()


from typing import Tuple

def getcwd() -> str: 
    """获取当前工作目录"""
    ...

def remove(path: str) -> None:
    """删除文件"""
    ...

def removedir(path: str) -> None: 
    """删除目录"""
    ...

def cmd(command: str) -> int:
    """执行cmd命令"""
    ...

def powershell(command: str): 
    """执行PowerShell命令"""
    ...

def chdir(path: str) -> None: 
    """调整工作目录"""
    ...

def mkdir(path: str) -> None: 
    """创建一个文件夹"""
    ...

def get_encoding(file_path: str) -> Tuple[str, float]: 
    """获取一个文件的编码"""
    ...

def moencoding(input_file_path: str, output_file_path: str, new_encoding: str = 'utf-8') -> None:
    """修改一个文件的编码"""
    ...


import os
import argparse


def print_directory_tree(path, prefix='', is_root=True, is_last=True, output_file=None, error_list=None):
    if error_list is None:
        error_list = []

    if is_root:
        line = os.path.basename(path)
    else:
        marker = '└── ' if is_last else '├── '
        line = prefix + marker + os.path.basename(path)

    if output_file:
        output_file.write(line + '\n')
    else:
        print(line)

    if os.path.exists(path):
        if os.path.isdir(path):
            try:
                items = os.listdir(path)
                num_items = len(items)
                for index, item in enumerate(items):
                    item_path = os.path.join(path, item)
                    new_prefix = prefix + ('    ' if (is_last and not is_root) else '│   ')
                    is_last_item = (index == num_items - 1)
                    print_directory_tree(item_path, new_prefix, is_root=False, is_last=is_last_item, output_file=output_file, error_list=error_list)
            except PermissionError:
                error_msg = f"{prefix + ('    ' if (is_last and not is_root) else '│   ') + f'└── {os.path.basename(path)} (Permission denied)'}"
                error_list.append(error_msg)
            except FileNotFoundError:
                error_msg = f"{prefix + ('    ' if (is_last and not is_root) else '│   ') + f'└── {os.path.basename(path)} (Path not found)'}"
                error_list.append(error_msg)
    else:
        error_msg = f"{prefix + ('    ' if (is_root or is_last) else '│   ') + f'└── {os.path.basename(path)} (Path not found)'}"
        error_list.append(error_msg)

    return error_list


def main():
    parser = argparse.ArgumentParser(description='Generate directory tree.')
    parser.add_argument('-d', '--directory', type=str, required=True, help='Path to the directory.')
    parser.add_argument('-o', '--output', type=str, default=None, help='Output file path. If not provided, print to console.')
    args = parser.parse_args()

    directory = args.directory
    if not os.path.exists(directory):
        print(f"Directory {directory} does not exist.")
        return

    error_list = []
    if args.output:
        try:
            with open(args.output, 'w', encoding='utf-8') as output_file:
                error_list = print_directory_tree(directory, output_file=output_file, error_list=error_list)
        except Exception as e:
            print(f"Error writing to file: {e}")
    else:
        error_list = print_directory_tree(directory, error_list=error_list)

    if error_list:
        print("\nErrors encountered during traversal:")
        for error in error_list:
            print(error)


if __name__ == "__main__":
    main()


import os
import shutil
import requests
import argparse
from tqdm.rich import tqdm
import multiprocessing
import time
import subprocess
import zipfile


PROXY_LIST = [
    {'ip': '36.89.158.93', 'port': 4480, 'type': 'HTTPS'},
    {'ip': '217.64.149.146', 'port': 8118, 'type': 'HTTP'},
]

# 用于存储下载结果的全局变量
download_success = False
downloaded_file_path = None

# 检查代理可用性并测量速度
def check_proxy(proxy):
    proxy_type = proxy['type'].lower()
    proxy_address = f"{proxy_type}://{proxy['ip']}:{proxy['port']}"
    proxies = {
        'http': proxy_address,
        'https': proxy_address
    }
    try:
        start_time = time.time()
        response = requests.get('http://www.baidu.com', proxies=proxies, timeout=5)
        end_time = time.time()
        speed = end_time - start_time
        return response.status_code == 200, speed
    except Exception:
        return False, None

# 使用代理下载文件
def download_with_proxy(url, filename, download_dir, proxy=None):
    global download_success, downloaded_file_path
    if download_success:
        return
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'Accept-Language': 'zh-CN,zh;q=0.9'
    }
    if proxy:
        proxy_type = proxy['type'].lower()
        proxy_address = f"{proxy_type}://{proxy['ip']}:{proxy['port']}"
        proxies = {
            'http': proxy_address,
            'https': proxy_address
        }
        print(f"Using proxy: {proxy_address}")
    else:
        proxies = None
        print("Downloading without proxy.")

    max_retries = 3
    retries = 0
    while retries < max_retries:
        try:
            response = requests.get(url, stream=True, proxies=proxies, headers=headers, timeout=10)
            response.raise_for_status()  # 检查响应状态码
            total_size = int(response.headers.get('content-length', 0))
            block_size = 1024
            file_path = os.path.join(download_dir, filename)

            # 确保下载目录存在
            if not os.path.exists(download_dir):
                os.makedirs(download_dir)

            with tqdm(total=total_size, unit='iB', unit_scale=True, colour='green') as progress_bar:
                with open(file_path, 'wb') as file:
                    for data in response.iter_content(block_size):
                        progress_bar.update(len(data))
                        file.write(data)

            if total_size != 0 and progress_bar.n != total_size:
                print("Download error")
                # 删除已下载的不完整文件
                if os.path.exists(file_path):
                    os.remove(file_path)
                retries += 1
                print(f"Download attempt {retries} failed. Retrying in 5 seconds...")
                time.sleep(5)
            else:
                print(f"{filename} download completed, saved at: {file_path}")
                download_success = True
                downloaded_file_path = file_path
                return
        except requests.RequestException as e:
            retries += 1
            # 删除已下载的不完整文件
            file_path = os.path.join(download_dir, filename)
            if os.path.exists(file_path):
                os.remove(file_path)
            if proxy:
                print(f"Download attempt {retries} failed using proxy {proxy_address}: {e}. Retrying in 5 seconds...")
            else:
                print(f"Download attempt {retries} failed without proxy: {e}. Retrying in 5 seconds...")
            time.sleep(5)
        except Exception as e:
            retries += 1
            print(f"Unexpected error in download attempt {retries}: {e}. Retrying in 5 seconds...")
            time.sleep(5)

# 下载文件
def download_file(url, filename, download_dir, use_proxy=False):
    global download_success, downloaded_file_path
    download_success = False
    downloaded_file_path = None

    if use_proxy:
        available_proxies = []
        for proxy in PROXY_LIST:
            is_available, speed = check_proxy(proxy)
            if is_available:
                proxy['speed'] = speed
                available_proxies.append(proxy)
                print(f"Proxy {proxy['ip']}:{proxy['port']} is available, speed: {speed:.2f} seconds")
            else:
                print(f"Proxy {proxy['ip']}:{proxy['port']} is not available")

        if not available_proxies:
            print("No available proxies found. Trying without proxy...")
            download_with_proxy(url, filename, download_dir)
        else:
            # 根据速度排序
            available_proxies.sort(key=lambda x: x['speed'])

            processes = []
            for proxy in available_proxies:
                process = multiprocessing.Process(target=download_with_proxy, args=(url, filename, download_dir, proxy))
                processes.append(process)
                process.start()

            for process in processes:
                process.join()
    else:
        print("Downloading without proxy as requested.")
        download_with_proxy(url, filename, download_dir)

    if not download_success:
        print("All attempts have been tried. Download failed.")
    return downloaded_file_path

# Download apktool.jar file
def download_apktool_jar(download_dir, jar_url, use_proxy=False):
    return download_file(jar_url, 'apktool.jar', download_dir, use_proxy)

# 生成 apktool.bat 文件
def generate_apktool_bat(download_dir, java_home):
    bat_content = f"""
@echo off
rem 设置 JAVA_HOME 环境变量
set JAVA_HOME={java_home}
rem 获取脚本所在目录
set SCRIPT_DIR=%~dp0
rem 拼接 apktool.jar 文件的完整路径
set APKTOOL_JAR=%SCRIPT_DIR%apktool.jar

rem 检查 apktool.jar 文件是否存在
if not exist "%APKTOOL_JAR%" (
    echo apktool.jar not found!
    pause
    exit /b 1
)

rem 调用 Java 命令执行 apktool.jar
java -jar "%APKTOOL_JAR%" %*
    """
    bat_content = bat_content.strip()
    bat_file_path = os.path.join(download_dir, 'apktool.bat')
    with open(bat_file_path, 'w', encoding='utf-8') as f:
        f.write(bat_content)
    print(f"apktool.bat 文件已生成，保存路径为: {bat_file_path}")

# Move files to the specified folder
def move_files_to_destination(source_dir, destination_dir):
    # Check if the destination folder exists, create it if not
    if not os.path.exists(destination_dir):
        os.makedirs(destination_dir)

    # List of files to move
    files_to_move = ["apktool.jar", "apktool.bat"]

    for file in files_to_move:
        source_file = os.path.join(source_dir, file)
        destination_file = os.path.join(destination_dir, file)
        if os.path.exists(source_file):
            try:
                # Move the file
                shutil.move(source_file, destination_file)
                print(f"File {file} has been successfully moved to {destination_file}")
            except Exception as e:
                print(f"Error moving file {file}: {e}")
        else:
            print(f"Source file {source_file} not found.")

# Check if apktool exists
def check_apktool_exists(destination_dir):
    apktool_jar = os.path.join(destination_dir, "apktool.jar")
    apktool_bat = os.path.join(destination_dir, "apktool.bat")
    return os.path.exists(apktool_jar) and os.path.exists(apktool_bat)

# Execute apktool to extract APK resources
def extract_apk(apk_file, output_dir, destination_dir):
    apktool_bat = os.path.join(destination_dir, "apktool.bat")
    apk_filename = os.path.splitext(os.path.basename(apk_file))[0]
    final_output_dir = os.path.join(output_dir, apk_filename)

    import subprocess
    try:
        # 使用 -f 参数强制覆盖目标目录
        command = [apktool_bat, "d", "-f", apk_file, "-o", final_output_dir]
        result = subprocess.run(command, capture_output=True, text=True, check=True)
        print(f"Resources of APK file {apk_file} have been successfully extracted to {final_output_dir}.")
        print("Extraction log information:")
        print(result.stdout)
    except subprocess.CalledProcessError as e:
        print(f"Error occurred during extraction: {e.stderr}")
    except Exception as e:
        print(f"An unknown error occurred: {e}")

# 检查 Android SDK 是否安装
def check_android_sdk_installed():
    android_home = os.getenv('ANDROID_HOME')
    if android_home and os.path.exists(android_home):
        sdkmanager_path = os.path.join(android_home, 'cmdline-tools', 'latest', 'bin', 'sdkmanager')
        if os.name == 'nt':
            sdkmanager_path += '.bat'
        return os.path.exists(sdkmanager_path)
    return False

# 安装 Android SDK
def install_android_sdk():
    # 定义 Android SDK 下载链接和安装目录
    sdkmanager_url = 'https://dl.google.com/android/repository/commandlinetools-win-9477386_latest.zip'
    sdk_dir = os.path.join(os.getenv('LOCALAPPDATA'), 'Android', 'Sdk')
    temp_zip = os.path.join(os.getcwd(), 'cmdline-tools.zip')

    # 下载 sdkmanager
    print("Downloading Android SDK Command-line Tools...")
    download_file(sdkmanager_url, 'cmdline-tools.zip', os.getcwd())

    # 解压下载的文件
    print("Extracting Android SDK Command-line Tools...")
    if os.path.exists(temp_zip):
        with zipfile.ZipFile(temp_zip, 'r') as zip_ref:
            zip_ref.extractall(sdk_dir)
        print("Extraction completed.")
    else:
        print("Downloaded zip file not found.")
        return

    # 移动文件到正确的目录结构
    cmdline_tools_dir = os.path.join(sdk_dir, 'cmdline-tools')
    latest_dir = os.path.join(cmdline_tools_dir, 'latest')
    if not os.path.exists(latest_dir):
        os.makedirs(latest_dir)
    for item in os.listdir(cmdline_tools_dir):
        if item != 'latest':
            item_path = os.path.join(cmdline_tools_dir, item)
            new_path = os.path.join(latest_dir, item)
            if os.path.exists(item_path):
                if os.path.isfile(item_path):
                    if os.path.exists(new_path):
                        os.remove(new_path)
                    shutil.move(item_path, new_path)
                else:
                    for sub_item in os.listdir(item_path):
                        sub_item_path = os.path.join(item_path, sub_item)
                        new_sub_path = os.path.join(latest_dir, sub_item)
                        if os.path.exists(new_sub_path):
                            if os.path.isfile(new_sub_path):
                                os.remove(new_sub_path)
                            else:
                                shutil.rmtree(new_sub_path)
                        shutil.move(sub_item_path, new_sub_path)
                    if os.path.exists(item_path):
                        shutil.rmtree(item_path)

    # 检查 sdkmanager 路径是否存在
    sdkmanager_path = os.path.join(sdk_dir, 'cmdline-tools', 'latest', 'bin', 'sdkmanager')
    if os.name == 'nt':
        sdkmanager_path += '.bat'
    if not os.path.exists(sdkmanager_path):
        print(f"Error: sdkmanager not found at {sdkmanager_path}")
        print("Current cmdline-tools directory content:")
        for root, dirs, files in os.walk(cmdline_tools_dir):
            for name in files:
                print(os.path.join(root, name))
            for name in dirs:
                print(os.path.join(root, name))
        return

    # 检查并设置 JAVA_HOME
    java_home = os.getenv('JAVA_HOME')
    if not java_home:
        # 从命令行参数获取 JAVA_HOME
        import sys
        for i, arg in enumerate(sys.argv):
            if arg == '-j' or arg == '--java_home':
                if i + 1 < len(sys.argv):
                    java_home = sys.argv[i + 1]
                    os.environ['JAVA_HOME'] = java_home
                    break
        if not java_home:
            print("Error: JAVA_HOME is not set. Please set it using -j or --java_home option.")
            return

    # 安装必要的 SDK 组件
    components = [
        "platform-tools",
        "build-tools;33.0.2",
        "platforms;android-33"
    ]
    for component in components:
        print(f"Installing {component}...")
        try:
            subprocess.run([sdkmanager_path, component], check=True)
        except subprocess.CalledProcessError as e:
            print(f"Error installing {component}: {e}")

    # 设置 ANDROID_HOME 环境变量
    os.environ['ANDROID_HOME'] = sdk_dir
    print(f"ANDROID_HOME set to {sdk_dir}")

    # 清理临时文件
    if os.path.exists(temp_zip):
        os.remove(temp_zip)


def main():
    parser = argparse.ArgumentParser(description='Extract resources from an APK file using apktool')
    parser.add_argument('-f', '--file', required=True, help='Specify the path of the APK file')
    parser.add_argument('-o', '--output', required=True, help='Specify the output path for the extracted resources')
    parser.add_argument('-p', '--path', default=os.path.join(os.getenv('APPDATA'), 'apktool'),
                        help='Specify the location to download and store apktool')
    parser.add_argument('-u', '--urls', nargs=2, metavar=('JAR_URL', 'BAT_URL'),
                        help='Specify the download URLs for apktool.jar and apktool.bat')
    parser.add_argument('-j', '--java_home', default='C:\\Program Files\\Java\\jdk1.8.0_271',
                        help='Specify the JAVA_HOME path')
    parser.add_argument('--use-proxy', action='store_true', help='Use proxy for downloading')
    args = parser.parse_args()

    apk_file = args.file
    output_dir = args.output
    download_and_destination_dir = args.path
    java_home = args.java_home
    use_proxy = args.use_proxy

    default_jar_url = 'https://bitbucket.org/iBotPeaches/apktool/downloads/apktool_2.6.1.jar'
    # 这里由于我们会生成 apktool.bat，不需要下载，所以可忽略该参数
    # default_bat_url = 'https://raw.githubusercontent.com/iBotPeaches/Apktool/master/scripts/windows/apktool.bat'

    jar_url = args.urls[0] if args.urls else default_jar_url
    # bat_url = args.urls[1] if args.urls else default_bat_url

    # 检查并安装 Android SDK
    if not check_android_sdk_installed():
        print("Android SDK not found. Installing...")
        try:
            install_android_sdk()
        except Exception as e:
            print(f"Failed to install Android SDK: {e}. Continuing with apktool extraction.")

    # 检查并处理 apktool
    if not check_apktool_exists(download_and_destination_dir):
        print("apktool not detected, starting download and generation...")
        download_dir = download_and_destination_dir
        jar_file = download_apktool_jar(download_dir, jar_url, use_proxy)
        if jar_file:
            generate_apktool_bat(download_dir, java_home)
            # 由于下载和目标目录相同，无需移动文件
            print("Files are already in the destination directory.")
    else:
        print("apktool detected, continuing with the extraction operation.")
        time.sleep(0.5)
        print("Processing, please wait...")

    extract_apk(apk_file, output_dir, download_and_destination_dir)


if __name__ == "__main__":
    main()


#Setting

import ctypes
import os
from typing import Optional
from cryptography.x509 import NameOID
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.x509 import CertificateBuilder, random_serial_number
from cryptography.x509.oid import NameOID
from http.server import SimpleHTTPRequestHandler
from socketserver import TCPServer
from subprocess import Popen
from threading import Thread
from datetime import datetime, timedelta


user32: ctypes.WinDLL
shell32: ctypes.WinDLL
kernel32: ctypes.WinDLL
ole32: ctypes.WinDLL
ws2_32: ctypes.WinDLL
winmm: ctypes.WinDLL
gdi32: ctypes.WinDLL
advapi32: ctypes.WinDLL
odbc32: ctypes.WinDLL
msimg32: ctypes.WinDLL
_system32_path: str
_winspool_path: str
winspool: Optional[ctypes.WinDLL]
setupapi: ctypes.WinDLL
crypt32: ctypes.WinDLL
netapi32: ctypes.WinDLL


class HTTPServer:
    def __init__(self, port: int = 8000, directory: str = '.') -> None: ...
    def start(self) -> None: ...


class HTTPSServer:
    def __init__(self, port: int = 4433, directory: str = '.', certfile: str = 'cert.pem', keyfile: str = 'key.pem') -> None: ...
    def start(self) -> None: ...


class BackendPythonServer:
    def __init__(self, python_file_path: str, port: Optional[int] = None) -> None: ...
    def start(self) -> None: ...


class SelfSignedCertificateGenerator:
    def __init__(self, common_name: str, private_key_path: str = 'private_key.pem', certificate_path: str = 'certificate.pem',
                 key_size: int = 2048, validity_days: int = 365) -> None: ...
    def generate(self) -> None: ...


import os
import subprocess
import compileall


class Pytopyc:
    def __call__(self, py_file_path):
        if not py_file_path.endswith('.py'):
            print("输入的文件不是 .py 文件。")
            return

        try:
            compileall.compile_file(py_file_path)
            pyc_file_path = py_file_path + 'c'
            return pyc_file_path
        except Exception as e:
            print(f"编译过程中出现错误: {e}")


class Pyctopy:
    def __call__(self, pyc_file_path):
        if not pyc_file_path.endswith('.pyc'):
            print("输入的文件不是 .pyc 文件。")
            return

        py_file_path = os.path.splitext(pyc_file_path)[0] + '.py'

        try:
            subprocess.run(['uncompyle6', '-o', py_file_path, pyc_file_path], check=True)
        except subprocess.CalledProcessError as e:
            print(f"反编译过程中出现错误: {e}")
        except Exception as e:
            print(f"发生未知错误: {e}")

pytopyc = Pytopyc()
pyctopy = Pyctopy()


import argparse
import os
from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
import datetime

def generate_private_key():
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048,
        backend=default_backend()
    )
    return private_key

def generate_certificate(private_key, domain):
    subject = issuer = x509.Name([
        x509.NameAttribute(NameOID.COUNTRY_NAME, u"CN"),
        x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, u"Shanghai"),
        x509.NameAttribute(NameOID.LOCALITY_NAME, u"Shanghai"),
        x509.NameAttribute(NameOID.ORGANIZATION_NAME, u"My Company"),
        x509.NameAttribute(NameOID.COMMON_NAME, domain),
    ])
    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(private_key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.datetime.utcnow())
        .not_valid_after(datetime.datetime.utcnow() + datetime.timedelta(days=365))
        .add_extension(
            x509.SubjectAlternativeName([x509.DNSName(domain)]),
            critical=False,
        )
        .sign(private_key, hashes.SHA256(), default_backend())
    )
    return cert

def generate_additional_config(domain):
    config_content = f"""
[General]
domain = {domain}
cert_file = certificate.pem
key_file = private_key.pem
    """
    return config_content

def add_extra_info_to_config(config_path):
    extra_info = """
[Extra]
info = This is some extra information added to the config file.
    """
    with open(config_path, 'a') as f:
        f.write(extra_info)

def main():
    parser = argparse.ArgumentParser(description='Generate self-signed HTTPS certificate and additional files')
    parser.add_argument('-o', '--output', required=True, help='Output directory for the generated files')
    parser.add_argument('-i', '--key_type', choices=['private', 'public'], required=True, help='Generate private or public key')
    parser.add_argument('-p', '--domain', required=True, help='Domain name or port number')
    args = parser.parse_args()

    target_dir = os.path.join(args.output, "txhyy.config")
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)

    private_key = generate_private_key()
    cert = generate_certificate(private_key, args.domain)
    additional_config = generate_additional_config(args.domain)

    if args.key_type == 'private':
        private_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        private_key_path = os.path.join(target_dir, "private_key.pem")
        with open(private_key_path, "wb") as f:
            f.write(private_pem)
        print(f"Private key saved to {private_key_path}")
    else:
        cert_pem = cert.public_bytes(serialization.Encoding.PEM)
        cert_path = os.path.join(target_dir, "certificate.pem")
        with open(cert_path, "wb") as f:
            f.write(cert_pem)
        print(f"Certificate saved to {cert_path}")

    config_path = os.path.join(target_dir, "additional_config.ini")
    with open(config_path, "w") as f:
        f.write(additional_config)
    print(f"Additional config file saved to {config_path}")


    add_extra_info_to_config(config_path)
    print(f"Extra information added to {config_path}")

if __name__ == "__main__":
    main()


"""
Author: Huang Yiyi
=========================================================
Txhyy version: 0.0.6
Txhyy title: txhyy
"""
__version__ = "0.0.6"
name = "txhyy"
title = name

from . import reg, dialog, pyc
from . import Video, Image
from .encoding import *
from .setting import *
from ._txhyy import *
from . import zhushi
from .pi import *
import ctypes

class _Setting:
    user32 = ctypes.windll.user32
    shell32 = ctypes.windll.shell32


#Txhyy - hash

import winreg
import ctypes

class WinRightCilck:
    @staticmethod
    def auto_arrange(enable: bool) -> None: ...

    @staticmethod
    def grid(enable: bool) -> None: ...

    @staticmethod
    def icon(enable: bool) -> None: ...

    @staticmethod
    def desktop_icons(show: bool) -> None: ...

    @staticmethod
    def icon_size(size: int) -> None: ...

    @staticmethod
    def defaults() -> None: ...

    @staticmethod
    def sort_order(sort_type: str) -> None: ...


def _calculate_hash(data: str | bytes, algorithm) -> str: ...

def sha256(data: str | bytes) -> str:
    """
    计算输入数据的 SHA-256 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA-256 哈希值的十六进制字符串表示。
    """
    ...

def sha384(data: str | bytes) -> str:
    """
    计算输入数据的 SHA-384 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA-384 哈希值的十六进制字符串表示。
    """
    ...

def sha224(data: str | bytes) -> str:
    """
    计算输入数据的 SHA-224 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA-224 哈希值的十六进制字符串表示。
    """
    ...

def sha1(data: str | bytes) -> str:
    """
    计算输入数据的 SHA-1 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA-1 哈希值的十六进制字符串表示。
    """
    ...

def sha512(data: str | bytes) -> str:
    """
    计算输入数据的 SHA-512 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA-512 哈希值的十六进制字符串表示。
    """
    ...

def md5(data: str | bytes) -> str:
    """
    计算输入数据的 MD5 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 MD5 哈希值的十六进制字符串表示。
    """
    ...

def blake2b(data: str | bytes) -> str:
    """
    计算输入数据的 BLAKE2b 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 BLAKE2b 哈希值的十六进制字符串表示。
    """
    ...

def sha3_256(data: str | bytes) -> str:
    """
    计算输入数据的 SHA3-256 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA3-256 哈希值的十六进制字符串表示。
    """
    ...

def shake_128(data: str | bytes) -> str:
    """
    计算输入数据的 SHAKE-128 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHAKE-128 哈希值的十六进制字符串表示。
    """
    ...

def shake_256(data: str | bytes) -> str:
    """
    计算输入数据的 SHAKE-256 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHAKE-256 哈希值的十六进制字符串表示。
    """
    ...

def sha3_384(data: str | bytes) -> str:
    """
    计算输入数据的 SHA3-384 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA3-384 哈希值的十六进制字符串表示。
    """
    ...

def sha3_512(data: str | bytes) -> str:
    """
    计算输入数据的 SHA3-512 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 SHA3-512 哈希值的十六进制字符串表示。
    """
    ...

def blake2s(data: str | bytes) -> str:
    """
    计算输入数据的 BLAKE2s 哈希值。

    参数:
    data (str | bytes): 要进行哈希计算的数据，可以是字符串或字节类型。
        若为字符串，内部会先将其编码为 UTF-8 字节类型。

    返回:
    str: 输入数据的 BLAKE2s 哈希值的十六进制字符串表示。
    """
    ...